// $Id: DRand48Engine.cpp,v 1.1.1.1 2000/04/03 21:03:44 wilko Exp $
// -*- C++ -*-
//
// -----------------------------------------------------------------------
//                             HEP Random
//                        --- DRand48Engine ---
//                      class implementation file
// -----------------------------------------------------------------------
// This file is part of Geant4 (simulation toolkit for HEP).

// =======================================================================
// G.Cosmo        - Created: 5th September 1995
//                - Minor corrections: 31st October 1996
//                - Added methods for engine status: 19th November 1996
//                - Added srand48(), seed48(), drand48() implementations
//                  for Windows/NT: 6th March 1997
//                - Fixed bug in setSeeds(): 15th September 1997
//                - Private copy constructor and operator=: 26th Feb 1998
// J.Marraffino   - Added stream operators and related constructor.
//                  Added automatic seed selection from seed table and
//                  engine counter: 16th Feb 1998
// =======================================================================

#include "CLHEP/Random/Random.h"
#include "CLHEP/Random/DRand48Engine.h"
//#include "CLHEP/String/Strings.h"

//#include <iostream>
//#include <fstream>
// Number of instances with automatic seed selection
HepInt DRand48Engine::numEngines = 0;

// Maximum index into the seed table
HepInt DRand48Engine::maxIndex = 215;

#ifdef WIN32
   // ********************************************************************
   // Code extracted from GNU C Library 2.0.1

#  include <limits.h>
#  include <sys/types.h>
#  include <wtypes.h>
#  include <string.h>

   /* Global state for non-reentrant functions.  */
   struct drand48_data libc_drand48_data;

   int drand48_iterate (unsigned short int xsubi[3], struct drand48_data *buffer)
   {
     ULONGLONG X, a, result;

     /* Initialize buffer, if not yet done.  */
     if (!buffer->init)
       {
   #if (USHRT_MAX == 0xffffU)
         buffer->a[2] = 0x5;
         buffer->a[1] = 0xdeec;
         buffer->a[0] = 0xe66d;
   #else
         buffer->a[2] = 0x5deecUL;
         buffer->a[1] = 0xe66d0000UL;
         buffer->a[0] = 0;
   #endif
         buffer->c = 0xb;
         buffer->init = 1;
       }

     /* Do the real work.  We choose a data type which contains at least
        48 bits.  Because we compute the modulus it does not care how
        many bits really are computed.  */

     if (sizeof (unsigned short int) == 2)
       {
                 X = (ULONGLONG)xsubi[2] << 32 | (ULONGLONG)xsubi[1] << 16 | xsubi[0];
         a = ((ULONGLONG)buffer->a[2] << 32 | (ULONGLONG)buffer->a[1] << 16
              | buffer->a[0]);

                 result = X * a + buffer->c;

         xsubi[0] = (unsigned short int) (result & 0xffff);
         xsubi[1] = (unsigned short int) ((result >> 16) & 0xffff);
         xsubi[2] = (unsigned short int) ((result >> 32) & 0xffff);
       }
     else
       {
         X = (ULONGLONG)xsubi[2] << 16 | xsubi[1] >> 16;
         a = (ULONGLONG)buffer->a[2] << 16 | buffer->a[1] >> 16;

         result = X * a + buffer->c;

         xsubi[0] = (unsigned short int) (result >> 16 & 0xffffffffl);
         xsubi[1] = (unsigned short int) (result << 16 & 0xffff0000l);
       }

     return 0;
   }

   int seed48_r (unsigned short int seed16v[3], struct drand48_data *buffer)
   {
     /* Save old value at a private place to be used as return value.  */
     memcpy (buffer->old_X, buffer->X, sizeof (buffer->X));

     /* Install new state.  */
     memcpy (buffer->X, seed16v, sizeof (buffer->X));

     return 0;
   }

   unsigned short int * seed48 (unsigned short int seed16v[3])
   {
     (void) seed48_r (seed16v, &libc_drand48_data);

     return libc_drand48_data.old_X;
   }

   int srand48_r (long seedval, struct drand48_data *buffer)
   {
     /* The standards say we only have 32 bits.  */
     if (sizeof (long) > 4)
       seedval &= 0xffffffffl;

   #if (USHRT_MAX == 0xffffU)
     buffer->X[2] = (unsigned short int) (seedval >> 16);
     buffer->X[1] = (unsigned short int) (seedval & 0xffffl);
     buffer->X[0] = 0x330e;
   #else
     buffer->X[2] = seedval;
     buffer->X[1] = 0x330e0000UL;
     buffer->X[0] = 0;
   #endif

     return 0;
   }
   
   void srand48 (long seedval)
   {
     (void) srand48_r (seedval, &libc_drand48_data);
   }

   int erand48_r (unsigned short int xsubi[3], struct drand48_data *buffer, double *result)
   {
     /* Compute next state.  */
     if (drand48_iterate (xsubi, buffer) < 0)
       return -1;

     /* Construct a positive double with the 48 random bits distributed over
        its fractional part so the resulting FP number is [0.0,1.0).  */

   #if USHRT_MAX == 65535
     *result = ((double) xsubi[2] / ((ULONGLONG)1 << 48) +
                    (double) xsubi[1] / ((ULONGLONG)1 << 32) +
                    (double) xsubi[0] / ((ULONGLONG)1 << 16));
   #else
   # error Unsupported size of short int
   #endif

     return 0;
   }

   double drand48 ()
   {
     double result;

     (void) erand48_r (libc_drand48_data.X, &libc_drand48_data, &result);

     return result;
   }

   // End Code extracted from GNU C Library 2.0.1
   // ********************************************************************
#endif  /* WIN32 */

DRand48Engine::DRand48Engine(long seed)
{
   setSeed(seed,0);
   setSeeds(&theSeed,0);
}

DRand48Engine::DRand48Engine()
{
   long seeds[2];
   long seed;
   HepInt cycle,curIndex;

   cycle = abs(HepInt(numEngines/maxIndex));
   curIndex = abs(HepInt(numEngines%maxIndex));
   numEngines += 1;
   long mask = ((cycle & 0x007fffff) << 8);
   HepRandom::getTheTableSeeds( seeds, curIndex );
   seed = seeds[0]^mask;
   setSeed(seed,0);
   setSeeds(&theSeed,0);
}

DRand48Engine::DRand48Engine(HepInt rowIndex, HepInt colIndex)
{
   long seed;
   long seeds[2];

   HepInt cycle = abs(HepInt(rowIndex/maxIndex));
   HepInt row = abs(HepInt(rowIndex%maxIndex));
   HepInt col = abs(HepInt(colIndex%2));
   long mask = ((cycle & 0x000007ff) << 20);
   HepRandom::getTheTableSeeds( seeds, row );
   seed = (seeds[col])^mask;
   setSeed(seed,0);
   setSeeds(&theSeed,0);
}

DRand48Engine::DRand48Engine(std::istream& is)
{
   is >> *this;
}

DRand48Engine::~DRand48Engine() {}

DRand48Engine::DRand48Engine(const DRand48Engine &p)
{
  // Assignment and copy of DRand48Engine objects may provoke
  // undesired behavior in a single thread environment.
  
  std::cerr << "!!! WARNING !!! - Illegal operation." << std::endl;
  std::cerr << "- Copy constructor and operator= are NOT allowed on "
	    << "DRand48Engine objects -" << std::endl;
  *this = p;
}

DRand48Engine & DRand48Engine::operator = (const DRand48Engine &p)
{
  // Assignment and copy of DRand48Engine objects may provoke
  // undesired behavior in a single thread environment.

  std::cerr << "!!! WARNING !!! - Illegal operation." << std::endl;
  std::cerr << "- Copy constructor and operator= are NOT allowed on "
	    << "DRand48Engine objects -" << std::endl;
  *this = p;
  return *this;
}

void DRand48Engine::setSeed(long seed, HepInt)
{
   srand48( seed );
   theSeed = seed;
}

void DRand48Engine::setSeeds(const long* seeds, HepInt)
{
  setSeed(seeds ? *seeds : 19780503L, 0);
  theSeeds = seeds;
}

void DRand48Engine::saveStatus( const char filename[] ) const
{
   std::ofstream outFile( filename, std::ios::out ) ;

   unsigned short dummy[] = { 0, 0, 0 };
   unsigned short* cseed = seed48(dummy);

   if (!outFile.bad()) {
     outFile << theSeed << std::endl;
     for (HepInt i=0; i<3; ++i) {
       outFile << cseed[i] << std::endl;
       dummy[i] = cseed[i];
     }
     seed48(dummy);
   }
}

void DRand48Engine::restoreStatus( const char filename[] )
{
   std::ifstream inFile( filename, std::ios::in);
   unsigned short cseed[3];

   if (!inFile.bad() && !inFile.eof()) {
     inFile >> theSeed;
     for (HepInt i=0; i<3; ++i)
       inFile >> cseed[i];
     seed48(cseed);
   }
}

void DRand48Engine::showStatus() const
{
   unsigned short dummy[] = { 0, 0, 0 };
   unsigned short* cseed = seed48(dummy);
   std::cout << std::endl;
   std::cout << "-------- DRand48 engine status ---------" << std::endl;
   std::cout << " Initial seed  = " << theSeed << std::endl;
   std::cout << " Current seeds = " << cseed[0] << ", ";
   std::cout                        << cseed[1] << ", ";
   std::cout                        << cseed[2] << std::endl;
   std::cout << "----------------------------------------" << std::endl;
   for (HepInt i=0; i<3; ++i)
     dummy[i] = cseed[i];
   seed48(dummy);
}

HepDouble DRand48Engine::flat()
{
   HepDouble num = 0.;

   while (num == 0.)
     num = drand48();
   return num;
}

void DRand48Engine::flatArray(const HepInt size, HepDouble* vect)
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i]=flat();
}

/*
std::ostream & operator<< ( std::ostream& os, const DRand48Engine& e  )
{
   unsigned short dummy[] = { 0, 0, 0 };
   unsigned short* cseed = seed48(dummy);
   const char *beginMarker = "DRand48Engine-begin";
   const char *endMarker = "DRand48Engine-end";

   os << " " << beginMarker << " ";
   os << e.theSeed << " ";
   for (HepInt i=0; i<3; ++i) {
     dummy[i] = cseed[i];
     os << cseed[i] << " ";
   }
   os << endMarker << " ";
   seed48(dummy);
   return os;
}
*/
/*
std::istream & operator >> ( std::istream& is, DRand48Engine& e )
{
   unsigned short cseed[3];
   const char * beginMarker = " ";
   const char * endMarker = " ";

   is >> std::ws >> beginMarker;
   if( beginMarker != "DRand48Engine-begin" ) {
     is.clear(std::ios::badbit | is.rdstate());
     std::cerr << "\nInput stream mispositioned or"
	       << "\nDRand48Engine state description missing or"
	       << "\nwrong engine type found." << std::endl;
     return is;
   }
   is >> e.theSeed;
   for (HepInt i=0; i<3; ++i) {
     is >> cseed[i];
   }
   is >> std::ws >> endMarker;
   if( endMarker != "DRand48Engine-end" ) {
     is.clear(std::ios::badbit | is.rdstate());
     std::cerr << "\nDRand48Engine state description incomplete."
	       << "\nInput stream is probably mispositioned now." << std::endl;
     return is;
   }
   seed48(cseed);
   return is;
}
*/
