// $Id: RandExponential.cpp,v 1.1.1.1 2000/04/03 21:03:45 wilko Exp $
// -*- C++ -*-
//
// -----------------------------------------------------------------------
//                             HEP Random
//                       --- RandExponential ---
//                      class implementation file
// -----------------------------------------------------------------------
// This file is part of Geant4 (simulation toolkit for HEP).

// =======================================================================
// Gabriele Cosmo - Created: 17th May 1996
//                - Added methods to shoot arrays: 28th July 1997
// J.Marraffino   - Added default mean as attribute and
//                  operator() with mean: 16th Feb 1998
// =======================================================================

#include "CLHEP/Random/RandExponential.h"

RandExponential::~RandExponential() {
  if ( deleteEngine ) delete localEngine;
}

RandExponential::RandExponential(const RandExponential& right)
 : defaultMean(right.defaultMean)
{;}

HepDouble RandExponential::operator()() {
  return fire( defaultMean );
}

HepDouble RandExponential::operator()( HepDouble mean ) {
  return fire( mean );
}

HepDouble RandExponential::shoot() {
  return -log(HepRandom::getTheEngine()->flat());
}

HepDouble RandExponential::shoot(HepDouble mean) {
  return -log(HepRandom::getTheEngine()->flat())*mean;
}

void RandExponential::shootArray( const HepInt size, HepDouble* vect,
                                  HepDouble mean )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = shoot(mean);
}

void RandExponential::shootArray(HepRandomEngine* anEngine, const HepInt size,
                                   HepDouble* vect, HepDouble mean )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = shoot(anEngine, mean);
}

void RandExponential::fireArray( const HepInt size, HepDouble* vect)
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = fire( defaultMean );
}

void RandExponential::fireArray( const HepInt size, HepDouble* vect,
                                 HepDouble mean )
{
   HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = fire( mean );
}
