// $Id: RandFlat.inl,v 1.1.1.1 2000/04/03 21:03:45 wilko Exp $
// -*- C++ -*-
// 
// -----------------------------------------------------------------------
//                            HEP Random
//                         --- RandFlat ---
//                 inlined functions implementation file
// -----------------------------------------------------------------------
// This file is part of Geant4 (simulation toolkit for HEP).

// =======================================================================
// Gabriele Cosmo - Created: 5th September 1995
// Peter Urban    - ShootBit() and related stuff added: 5th Sep 1996
// Gabriele Cosmo - Additional methods to fill arrays specifying
//                  boundaries: 24th Jul 1997 
//                - Fixed bug in shootInt(m,n): 25th Sep 1997
// J.Marraffino   - Added default arguments as attributes: 16th Feb 1998
// =======================================================================

inline RandFlat::RandFlat(HepRandomEngine & anEngine)
: localEngine(&anEngine), deleteEngine(false), firstUnusedBit(0),
  defaultWidth(1.0), defaultA(0.0), defaultB(1.0) {}

inline RandFlat::RandFlat(HepRandomEngine & anEngine, HepDouble width )
: localEngine(&anEngine), deleteEngine(false), firstUnusedBit(0),
  defaultWidth(width), defaultA(0.0), defaultB(width) {}

inline RandFlat::RandFlat(HepRandomEngine & anEngine, HepDouble a,
                                                      HepDouble b )
: localEngine(&anEngine), deleteEngine(false), firstUnusedBit(0),
  defaultWidth(b-a), defaultA(a), defaultB(b) {}

inline RandFlat::RandFlat(HepRandomEngine * anEngine)
: localEngine(anEngine), deleteEngine(true), firstUnusedBit(0),
  defaultWidth(1.0), defaultA(0.0), defaultB(1.0) {}

inline RandFlat::RandFlat(HepRandomEngine * anEngine, HepDouble width )
: localEngine(anEngine), deleteEngine(false), firstUnusedBit(0),
  defaultWidth(width), defaultA(0.0), defaultB(width) {}

inline RandFlat::RandFlat(HepRandomEngine * anEngine, HepDouble a,
                                                      HepDouble b )
: localEngine(anEngine), deleteEngine(false), firstUnusedBit(0),
  defaultWidth(b-a), defaultA(a), defaultB(b) {}

inline HepDouble RandFlat::shoot(HepDouble a, HepDouble b) {
  return (b-a)* shoot() + a;
}

inline HepDouble RandFlat::shoot(HepDouble width) {
  return width * shoot();
}

inline long RandFlat::shootInt(long n) {
  return long(shoot()*HepDouble(n));
}

inline long RandFlat::shootInt(long m, long n) {
  return long(shoot()*HepDouble(n-m)) + m;
}

inline void RandFlat::shootBits() {
  const HepDouble factor= 2.0*MSB; // this should fit into a double! 
  staticFirstUnusedBit= MSB;
  staticRandomInt= (unsigned long)(factor*shoot());  
}

inline HepInt RandFlat::shootBit() {
  if (staticFirstUnusedBit==0)
    shootBits();
  unsigned long temp= staticFirstUnusedBit&staticRandomInt;
  staticFirstUnusedBit>>= 1;
  return temp!=0;   
}

//---------------------

inline HepDouble RandFlat::shoot(HepRandomEngine* anEngine) {
  return anEngine->flat();
}


inline HepDouble RandFlat::shoot(HepRandomEngine* anEngine,
                                 HepDouble a, HepDouble b) {
  return (b-a)* anEngine->flat() + a;
}

inline HepDouble RandFlat::shoot(HepRandomEngine* anEngine,
                                 HepDouble width) {
  return width * anEngine->flat();
}

inline long RandFlat::shootInt(HepRandomEngine* anEngine,
                                  long n) {
  return long(anEngine->flat()*HepDouble(n));
}

inline long RandFlat::shootInt(HepRandomEngine* anEngine,
                                  long m, long n) {
  return long(HepDouble(n-m)*anEngine->flat()) + m;
}

inline void RandFlat::shootArray(HepRandomEngine* anEngine,
                                 const HepInt size, HepDouble* vect) {
  anEngine->flatArray(size,vect);
}

inline void RandFlat::shootBits(HepRandomEngine* engine) {
  const HepDouble factor= 2.0*MSB; // this should fit into a double! 
  staticFirstUnusedBit= MSB;
  staticRandomInt= (unsigned long)(factor*shoot(engine));  
}

inline HepInt RandFlat::shootBit(HepRandomEngine* engine) {
  if (staticFirstUnusedBit==0)
    shootBits(engine);
  unsigned long temp= staticFirstUnusedBit&staticRandomInt;
  staticFirstUnusedBit>>= 1;
  return temp!=0;   
}

//---------------------

inline HepDouble RandFlat::fire() {
  return (defaultB-defaultA)*localEngine->flat()+defaultA;
}

inline HepDouble RandFlat::fire(HepDouble a, HepDouble b) {
  return (b-a)* localEngine->flat() + a;
}

inline HepDouble RandFlat::fire(HepDouble width) {
  return width * localEngine->flat();
}

inline long RandFlat::fireInt(long n) {
  return long(localEngine->flat()*HepDouble(n));
}

inline long RandFlat::fireInt(long m, long n) {
  return long(localEngine->flat()*HepDouble(n-m)) + m;
}

inline void RandFlat::fireBits() {
  const HepDouble factor= 2.0*MSB; // this should fit into a HepDouble! 
  firstUnusedBit= MSB;
  randomInt= (unsigned long)(factor*localEngine->flat());  
}

inline HepInt RandFlat::fireBit() {
  if (firstUnusedBit==0)
    fireBits();
  unsigned long temp= firstUnusedBit&randomInt;
  firstUnusedBit>>= 1;
  return temp!=0;   
}
