// $Id: RandGeneral.cpp,v 1.1.1.1 2000/04/03 21:03:45 wilko Exp $
// -*- C++ -*-
//
// -----------------------------------------------------------------------
//                             HEP Random
//                          --- RandGeneral ---
//                      class implementation file
// -----------------------------------------------------------------------

// =======================================================================
// S.Magni & G.Pieri - Created: 5th September 1995
// G.Cosmo           - Added constructor using default engine from the
//                     static generator. Simplified shoot() and
//                     shootArray() (not needed in principle!): 20th Aug 1998
// M.G.Pia & G.Cosmo - Fixed bug in computation of theIntegralPdf in
//                     two constructors: 5th Jan 1999
// S.Magni & G.Pieri - Added linear interpolation: 24th Mar 1999
// =======================================================================

#include "CLHEP/Random/RandGeneral.h"

//////////////////
// Constructors
//////////////////

RandGeneral::RandGeneral( HepDouble* aProbFunc, HepInt theProbSize, HepInt IntType  )
  : deleteEngine(false), nBins(theProbSize), InterpolationType(IntType)
{
  localEngine = HepRandom::getTheEngine();
  register HepInt ptn;
  theIntegralPdf = new HepDouble[theProbSize+1];
  theIntegralPdf[0] = 0;
  for ( ptn = 0; ptn<theProbSize; ++ptn ) {
    theIntegralPdf[ptn+1] = theIntegralPdf[ptn] + aProbFunc[ptn];
  } 
  for ( ptn = 0; ptn < theProbSize+1; ++ptn )
    theIntegralPdf[ptn] /= theIntegralPdf[nBins];
}

RandGeneral::RandGeneral(HepRandomEngine& anEngine,
                         HepDouble* aProbFunc, HepInt theProbSize, HepInt IntType  )
: localEngine(&anEngine), deleteEngine(false), nBins(theProbSize),InterpolationType(IntType)
{
  register HepInt ptn;
  theIntegralPdf = new HepDouble[theProbSize+1];
  theIntegralPdf[0] = 0;
  for ( ptn = 0; ptn<theProbSize; ++ptn ) {
    theIntegralPdf[ptn+1] = theIntegralPdf[ptn] + aProbFunc[ptn];
  } 
  for ( ptn = 0; ptn < theProbSize+1; ++ptn )
    theIntegralPdf[ptn] /= theIntegralPdf[nBins];
}

RandGeneral::RandGeneral(HepRandomEngine* anEngine,
                         HepDouble* aProbFunc, HepInt theProbSize, HepInt IntType )
: localEngine(anEngine), deleteEngine(true), nBins(theProbSize),InterpolationType(IntType)
{
  register HepInt ptn;
  theIntegralPdf = new HepDouble[theProbSize+1];
  theIntegralPdf[0] = 0.;
  for ( ptn = 0; ptn<theProbSize; ++ptn ) {
    theIntegralPdf[ptn+1] = theIntegralPdf[ptn] + aProbFunc[ptn];
  } 
  for ( ptn = 0; ptn < theProbSize+1; ++ptn )
    theIntegralPdf[ptn] /=  theIntegralPdf[nBins-1];
}

//////////////////
//  Destructor
//////////////////

RandGeneral::~RandGeneral() {
  if ( deleteEngine ) delete localEngine;
  delete theIntegralPdf;
}

HepDouble RandGeneral::operator()() {
  return fire();
}
 
HepDouble RandGeneral::shoot( HepRandomEngine* anEngine )
{
  HepDouble rand;
  HepInt nabove, nbelow = 0, middle;
  
  nabove = nBins;  
  rand = anEngine->flat();
  
  while(nabove-nbelow > 1) {
    middle = ( nabove + nbelow ) / 2;
    if (rand == theIntegralPdf[middle]) break;
    if (rand < theIntegralPdf[middle]) nabove = middle;
    else nbelow = middle;
  }
  if ( InterpolationType == 1 ) {

    return ((HepDouble)nbelow) / (nBins);

  } else if ( InterpolationType == 0 ) {

    HepDouble x1,x2,y1,y2,m,q;
  
    x1 = HepDouble ( nbelow );
    x2 = HepDouble ( nbelow+1 );
    y1 = theIntegralPdf[nbelow];
    y2 = theIntegralPdf[nbelow+1];
    m = (y2-y1)/(x2-x1);
    q = y1 - m*x1;
    return ( ((rand - q)/m)/HepDouble(nBins));
 
  }
  else return 0;
}

void RandGeneral::shootArray( HepRandomEngine* anEngine,
                            const HepInt size, HepDouble* vect )
{
   register HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = shoot(anEngine);
}

HepDouble RandGeneral::fire()
{
  HepDouble rand;
  HepInt nabove, nbelow = 0, middle;
  
  nabove = nBins;  
  rand = localEngine->flat();
  
  while(nabove-nbelow > 1) {
    middle = ( nabove + nbelow ) / 2;
    if (rand == theIntegralPdf[middle]) break;
    if (rand < theIntegralPdf[middle]) nabove = middle;
    else nbelow = middle;
  }
  if ( InterpolationType == 1 ) {

    return ((HepDouble)nbelow) / (nBins);

  } else if ( InterpolationType == 0 ) {

    HepDouble x1,x2,y1,y2,m,q;
  
    x1 = HepDouble ( nbelow );
    x2 = HepDouble ( nbelow+1 );
    y1 = theIntegralPdf[nbelow];
    y2 = theIntegralPdf[nbelow+1];
    m = (y2-y1)/(x2-x1);
    q = y1 - m*x1;
    return ( ((rand - q)/m)/HepDouble(nBins));
 
  }
  else return 0;
}

void RandGeneral::fireArray( const HepInt size, HepDouble* vect )
{
   register HepInt i;

   for (i=0; i<size; ++i)
     vect[i] = fire();
}
