#ifndef algConditionalTask_h
#define algConditionalTask_h

#include "Event/algorithmConditional.h"
#include "Event/serviceI.h"

//-----------------------------
//! algorithmTask is algorithmConditional with the services of serviceI.
/*!
\e algorithmTask is an algorithm.<br>
\e algorithmTask is a conditional algorithm run only if a cut condition is fulfill<br>
\e algorithmTask uses the services of serviceI.<br>
an external client (optionManager) can set dinamically the algorithm and 
the cut to be run by the algorithmConditional of this algorithmTask.
<ul>
	<li> All algConditionalTask should have a unique name. 
	<li> algConditionalTask receives the services of serviceI using the \c setName() method.
	<li> An external user (optionManager) can set the algorithm and the cut of the algorithmConditional
	via the external options. It uses \c setOption() method inherited from optionVI.
</ul>
*/
//-----------------------------
// JA Hernando, Santa Cruz, CA 06/16/00
//-----------------------------
//##############################
class algConditionalTask : public algorithmConditional , public serviceI
//##############################
{
public:

	//! default contructor - set the name
	algConditionalTask(std::string name) {m_cutName="",m_algName="";setName(name);}
	//! defualt destructor
	virtual ~algConditionalTask(){}

	//! writeOut information about this class
	virtual void writeOut() const;

protected:
	
	//! define the parameters options to set the algorithm and the cut
	virtual void defineOption();

	//! set the options: add the cut or the algorithm into the algorithmConditional
	virtual void setOption(std::string par, std::string content);

private:

	//! dummy - to set the cut name
	std::string m_cutName;
	//! dummy - to set the algorithm name
	std::string m_algName;
};
#endif