#ifndef converterServer_h
#define converterServer_h

#include "Event/serverVI.h"
#include "Event/converterVI.h"

//-----------------------------------
//! protected server of converters. 
//-----------------------------------
/*!
\e converterServers stores a collection of user defined converters and turns
them into load/save algorithms
<ul> 
	<li> A derived class should create and add the converters into the server.
	(via the pure virtual method \c defineConverters()).
	<li> The converters are added into the processManager algorithm server via
	the method \c makeAlgorithms().
	<li> The \c load() and \c save() method allows to transform from
	persistent to transient data or viceversa.
	<li> Recomended: the name of the converters should be the name of transient data class.  
</ul>
*/
//-----------------------------------
// JA Hernando, Santa Cruz, 06/16/00
//-----------------------------------
//###################################
class converterServer
//###################################
{
protected:

	//! constructor - call to define the converters, makes the algorithms
	converterServer();
	//! default destructor
	virtual ~converterServer() {}

	//! load (Persistent->Transient) a named class 
	void load(std::string name) {getConverter(name)->load();}
	//! save (transiente->persistent) a named class
	void save(std::string name) {getConverter(name)->save();}

	//! add converter into the server
	virtual void  addConverter(std::string name, converterVI* conv);
	//! returns the converter of a given name
	virtual converterVI* getConverter(std::string name);

	//! virtual function to define the coverters to the server
	/*! the method \c makeAlgorithms() should be executed after the definition of the converters */
	virtual void defineConverters() {makeAlgorithms();};

	//! make algorithms from the converters
	void makeAlgorithms();

private:

	//! server with the list of conveters
	serverVI<converterVI>* m_server;
};
#endif