#ifndef optionServer_h
#define optionServer_h

#include "Event/serviceI.h"
#include "Event/serverVI.h"

//------------------------------------------------
//! Stores and applies options to a collection of optionVI derived classes.
/*! 
\e OptionServer sets options to classed derived from optionVI and previously declared 
into its server.<br>
\e OptionServer reads and sets options from an external file.<br>
<ul>
	<li> The options can be set to classes derived from optionVI.They should be declared
	into the server, using the method \c addToServer().
	Every object should have a unique name into the server.
	<li> The \c setOption() method allows to set a collection of options from a external
	file. The options should be listed in the file in the following way: 
	\e "objectName parName parType parContent",
	where \e parType can be "S" for string, "D" for double or "I" for integer.
	A line started with \e "//" in the file will be ignored and it can be use to put comments).
	<li> Use the method \c setOptionFileName() to set the name of the file with 
	the list of options to apply.
	<li> Options can be apply from a external client via the methods \c setOption(objectName, parName, parContent).
	The option is called parName and the content is parContent. 
</ul>
*/
//----------------------------------------------
//             J.A Hernando, Santa Cruz 02/29/00
//----------------------------------------------

//##############################
class optionServer 
//##############################
{
	friend class optionManager;

public:

	//! constructor
	optionServer();
	//! destructor
	virtual ~optionServer();

	void setOptionFileName(std::string name)   {m_optionFileName = name;}
	//! set a collection of options from a external file (set in m_optionFileName);
	void setOption();

	//! virtual method that returns true if the named object is in the server
	virtual bool search(std::string name) const {return m_server->search(name);}
	//! method to include a optionVI class in the server
	virtual void addToServer(std::string name, optionVI* op);

	//! method to writeOut the option information of all the classes in the server
	virtual void writeOutOptionsServer() const;
	//! metho to writeOut information about this class
	virtual void writeOut() const;

protected:

	//! returns the optionVI of the object with name
	optionVI* getOption(std::string name) const;

	//! set a string option in an object named className inserted in the server
	void setOption(std::string className, std::string par, std::string con);
	//! set a double option in an object named className
	void setOption(std::string className, std::string par, double d);
	//! set a int option in an object named className
	void setOption(std::string className, std::string par, int i);

protected:

	//! default ASCII file with the list of options
	std::string m_optionFileName;

	//! Server with the classes where optionVI can be apply()
	serverVI<optionVI>* m_server;

};


#endif