#ifndef OPTIONVI_H
#define OPTIONVI_H

#include "Event/nameVI.h"
#include "Event/serverVI.h"

#include <string>
//----------------------------------------------------
//! Interface to set options externally into a derived class
/*! 
<ul>
	<li> The derived class can declare some of its parameters as options. The parameters
	should be strings, doubles, integeres or  bool. The derived class should rewrite the virtual
	method \c defineOption() to include the \c defineOption(parName, parPointer) methods
	with the parameter name and the pointer to this parameter. Usually \c defineOptions()
	is just a collection of \c defineOption(parname, parpointer).
	<li> The derived class or a external friend class (optionServer) can set the value
	of options via the methods \c setOption(parName, parContent). The options are identified
	with their names.
	<li> Note that the methods \c setOption() are not protected and they not returns information of errors. 
	The external class that set the options should handle that.
</ul>
*/
//----------------------------------------------
//             J.A Hernando, Santa Cruz 02/29/00
//----------------------------------------------

//############################
class optionVI : public nameVI
//############################
{
	friend class optionServer;

public:

	//! constructor 
	optionVI() {clear();};
	//! default destructor
	virtual ~optionVI() {}

protected:

	//! add the derived object into the optionManager
	virtual void setName(std::string name);

	//! returns true if an option with that name is in one of the servers
	bool search(std::string name) const;

	//! virtual method to be overwriten to define the options of the derived class
	virtual void defineOption() {};
	
	//! define a string parameter of the derived class as option string
	void defineOption(std::string name, std::string* st) {m_sserver.add(name,st);}
	//! define a double parameter of the derived class as option double
	void defineOption(std::string name, double* d) {m_dserver.add(name,d);}
	//! define a integer paramter of the derived class as option integer
	void defineOption(std::string name, int* i) {m_iserver.add(name,i);}

	//! virtual method to set the option in a string parameter
	virtual void setOption(std::string name, std::string c) {*(m_sserver.get(name)) = c;}
	//! virtual method to set the option in a double parameter
	virtual void setOption(std::string name, double c) {*(m_dserver.get(name)) = c;}
	//! virtual method to set the option in a integer/bool parameter
	virtual void setOption(std::string name, int c) {*(m_iserver.get(name)) = c;}

	//! virtual method to writeOut the option defined in this class
	virtual void writeOutOptions() const;

	//! clear the servers
	virtual void clear() {m_sserver.clear();m_dserver.clear();m_iserver.clear();}

private:

	//! string option server
	serverVI<std::string> m_sserver;
	//! double option server
	serverVI<double> m_dserver;
	//! integer option server
	serverVI<int> m_iserver;

};
#endif
