#ifndef trsDataVI_H
#define trsDataVI_H

#include <string>
#include <iostream>

//----------------------------------------------
//! virtual bare class for the transient data
/*! 
The transient data of the program should inherit from this base class.<br>
The transient data assumes:
<ul>
	<li> it will be created one (execute the \c ini() method)
	<li> the pointer will be preserved alive and that the data will be cleared up and filled
again. 
</ul>
<ul>
	<li> trsDataVI has a main virtual method update() that executes 
	the pure virtual methods clear() and make(). The clear() and make() should
	be implemented by the derived class. In principle, clear() clear the data
	and make() makes or fill the data. <br> Note that this method is usually null and
	replaced by the action of an algorithmVI.
	<li> There is an pure virtual method ini() to initialize the data. Most of the data is created only one,
	the pointer kept in a serverVI, and the contents of the data replaced
	every update().
	<li> For convenience we add a virtual
	method writeOut() to print out the information of the data. To be compatible
	with the messageVI writeOut() method.
	<li> The command() method executed the method of this class via their names.
	It is useful to create composite classes. It is used for example in the
	template class serverVI.
</ul>
*/
//----------------------------------------------
//  J.A Hernando, Santa Cruz,CA 02/29/00
//----------------------------------------------
//###########################
class trsDataVI 
//###########################
{
public:



	//! default constructor
	trsDataVI() {}
	//! default destructor
	virtual ~trsDataVI() {}

	//! virtual method to clear and make/fill the transient data
 	virtual void update();

	//! pure virtual method to initialize the data (pointers if needed)
	virtual void ini()   = 0;
	//! pure virtual method to clear the data
	virtual void clear() = 0;
	//! pure virtual method to make/fill the data. 
	/*! usually this method is replaced by the action of an algorithmVI */
	virtual void make()   = 0;

	//! virtual function to writeOut the contents of the derived class
	/*! note that has the same name that the method in messageVI*/
	virtual void writeOut() const {};

	//! virtual method to execute method of this class for composites
	/*! It can be use for example in serverVI <trsData> classes*/
	virtual void command(std::string com);

};
#endif
