#include "RootTree/BeamTaggerHistoFolder.h"

#include "RootTree/rTreeRawServer.h"
#include "Event/dataManager.h"
#include "generation/BeamTagger.h"

#include "TFile.h"
#include "TH1.h"
#include "TH2.h"
#include "TNtuple.h"
#include "TROOT.h"

#include "TMath.h"
#include "TObject.h"
#include "TObjArray.h"
#include "TMap.h"

//###############################
void BeamTaggerHistoFolder::define() 
//###############################
{	
	if (!defined()) return;
	ntupleDefine();
	m_tagger = dataManager::instance()->getData("BeamTagger",m_tagger); 
	setDefine(true);
}

//###############################
void BeamTaggerHistoFolder::fill() 
//###############################
{	
	if (!defined()) define();
	ntupleFill();
	setDefine(true);

}
//###############################
void BeamTaggerHistoFolder::ntupleDefine()
//###############################
{	
	getFile()->cd();

	char* varname = "IdEvent:CleanBeam:XBeam:YBeam:nParticle:particleID:Momentum";
	TNtuple* ntTagger = new TNtuple("ntTagger", "Beam/Tagger ntuple", varname);
}
//###############################################
void BeamTaggerHistoFolder::ntupleFill()
//###############################################
{
	getFile()->cd();

	Float_t idEvent = dataManager::instance()->IO()->treeIn()->eventNumber();
	Float_t clean   = m_tagger->clean();
	Float_t Xbeam   = m_tagger->XBeam();
	Float_t Ybeam   = m_tagger->YBeam();
	Float_t nPart   = m_tagger->nParticle();

	for (int i = 0; i < (int)nPart; i++) {
		Float_t pid = m_tagger->particleID(i);
		Float_t mom = m_tagger->momentum(i);

		((TNtuple*) getFile()->Get("ntTagger"))->Fill(
			idEvent,clean,Xbeam,Ybeam,nPart,pid,mom);
	}
}