
#include "RootTree/CalHistoFolder.h"

#include "TFile.h"
#include "TH1.h"
#include "TH2.h"
#include "TNtuple.h"
#include "TROOT.h"

#include "TMath.h"
#include "TObject.h"
#include "TObjArray.h"
#include "TMap.h"

#include "instrument/CalBase.h"
#include "Event/dataManager.h"
#include "RootTree/rTreeRawServer.h"
#include "instrument/CalADCLogs.h"
#include "reconstruction/CalRecLogs.h"
#include "reconstruction/CsIClusters.h"

//###############################
void CalHistoFolder::define() 
//###############################
{	
	if (defined()) return;

	m_cls = dataManager::instance()->getData("CsIClusterList",m_cls);
	m_adc = dataManager::instance()->getData("CalRawLogs",m_adc);
	m_rec = dataManager::instance()->getData("CalRecLogs",m_rec);

	histosDefine(m_cls);

	ntupleDefine(m_adc);
	ntupleDefine(m_rec);
	ntupleDefine(m_cls);
	setDefine(true);
}
//###############################
void CalHistoFolder::fill() 
//###############################
{	
	if (!defined()) define();

	histosFill(m_cls);

	ntupleFill(m_adc);
	ntupleFill(m_rec);
	ntupleFill(m_cls);

}
//--------- define ---------------
//###############################
void CalHistoFolder::histosDefine(CsIClusterList* s) 
//###############################
{
	getFile()->cd();

	TH1F* htProfile = new TH1F("htProfile"," mean shower profile ",8,0,8); 
}	
//###############################
void CalHistoFolder::ntupleDefine(CalADCLogs* lg) 
//###############################
{
	getFile()->cd();

	char* varname = "IdEvent:Id:Side:Range:ADC:nHits";
	TNtuple* ntADCLogs = new TNtuple("ntADCLogs", "ntACDLogs ntuple",varname);

}
//###############################
void CalHistoFolder::ntupleDefine(CsIClusterList* cls) 
//###############################
{
	getFile()->cd();

	char* varname ="nHits:etot:e0:e1:e2:e3:e4:e5:e6:e7"
		           ":Fit_Energy:Chisq:Alpha:Lambda:Start"
					":x0:x1:x2:x3:x4:x5:x6:x7"
					":y0:y1:y2:y3:y4:y5:y6:y7";
	TNtuple* ntEneLayer = new TNtuple("ntEneLayer", " Energy in layers",varname);
}
//###############################
void CalHistoFolder::ntupleDefine(CalRecLogs* lg) 
//###############################
{
	getFile()->cd();

	char* varname = "IdEvent:Id:eneNeg:enePos:ene:etot:nHits:x:y";
	TNtuple* ntRecLogsEne = new TNtuple("ntRecLogsEne", "ntRecLogs Ene ntuple",varname);

	char* varname2 = "IdEvent:Id:etot:nHits"
			":ePosLEX:ePosLE:ePosHEX:ePosHE"
			":eNegLEX:eNegLE:eNegHEX:eNegHE";
	TNtuple* ntRecLogsRan = new TNtuple("ntRecLogsRan", "ntRecLogs Range ntuple",varname2);

	char* varname3 = "IdEvent:Id:etot:nHits"
		":ADCPosLEX:ADCPosLE:ADCPosHEX:ADCPosHE"
		":ADCNegLEX:ADCNegLE:ADCNegHEX:ADCNegHE";


	TNtuple* ntRecLogsADC = new TNtuple("ntRecLogsADC", "ntRecLogs ADC ntuple", varname3);

}
//###############################################
void CalHistoFolder::histosFill(CsIClusterList* cls)
//###############################################
{
	getFile()->cd();

	int nClust = cls->num();
	for ( int icl = 0; icl<nClust;icl++){
	CsICluster* cl = cls->Cluster(icl);
	const std::vector<double>& e = cl->getEneLayer();

	for(int layer = 0; layer<8; layer++)
	((TH1F*) getFile()->Get("htProfile"))->Fill(layer+0.5,e[layer]); 
	}
}
//###############################################
void CalHistoFolder::ntupleFill(CalADCLogs* adcLogs)
//###############################################
{
	getFile()->cd();

	SiLayers* siLayers = 0;
	siLayers = dataManager::instance()->getData("SiLayers",siLayers);
	Float_t nStrips = siLayers->numStrips();
	Float_t ide = dataManager::instance()->IO()->treeIn()->eventNumber();
	unsigned int nLogs = adcLogs->num();

	unsigned int i = 0;
	for (i = 0; i < nLogs; i++) {
		CalADCLog* adcLog = adcLogs->Log(i);
		Float_t idEvent  = ide;
		Float_t id       = adcLog->logID(); 
		for (int iside = 0; iside < CALNSIDES; iside++) {
			Float_t idSide = iside;
			for (int irange = 0; irange < CALNRANGES; irange++) {
				Float_t idRange = irange;
				Float_t ADC = adcLog->ADC(CalBase::side(iside),CalBase::range(irange));
				((TNtuple*) getFile()->Get("ntADCLogs"))->Fill(idEvent,id,
					idSide,idRange,ADC,nStrips);
			}
		}
	}
}
//###############################################
void CalHistoFolder::ntupleFill(CsIClusterList* cls)
//###############################################
{
	getFile()->cd();

	SiLayers* siLayers = 0;
	siLayers = dataManager::instance()->getData("SiLayers",siLayers);
	Float_t nStrips = siLayers->numStrips();
		
	int nClust = cls->num();
	for ( int icl = 0; icl<nClust;icl++){
	CsICluster* cl = cls->Cluster(icl);
	const std::vector<double>& eneLayer = cl->getEneLayer();
	const std::vector<Vector>& posLayer = cl->getPosLayer();
	double fit_ener = cl->getFitEnergy();
	double profchi2 = cl->getProfChisq();
	double fitalpha = cl->getCsiAlpha();
	double fitlambda = cl->getCsiLambda();
	double start = cl->getCsiStart();
	float ntarg[31];
	ntarg[0] = nStrips;
	ntarg[1] = cl->energySum();
	for(int i = 0; i <8; i++){
		ntarg[i+2] = eneLayer[i];
		ntarg[i+15] = (posLayer[i]).x();
		ntarg[i+23] = (posLayer[i]).y();
	}
	ntarg[10] = fit_ener;
	ntarg[11] =	profchi2;
	ntarg[12] =	fitalpha;
	ntarg[13] =	fitlambda;
	ntarg[14] =	start;



	((TNtuple*) getFile()->Get("ntEneLayer"))->Fill(ntarg);
//		Fill(nStrips,etot,e[0],e[1],e[2],e[3],e[4],e[5],e[6],e[7],fit_ener,profchi2,fitalpha,fitlambda,start);

	}
}
//###############################################
void CalHistoFolder::ntupleFill(CalRecLogs* recLogs)
//###############################################
{
	getFile()->cd();

	Float_t ide = dataManager::instance()->IO()->treeIn()->eventNumber();
	unsigned int nLogs = recLogs->num();

	SiLayers* siLayers = 0;
	siLayers = dataManager::instance()->getData("SiLayers",siLayers);
	Float_t nStrips = siLayers->numStrips();
	CsIClusterList* calCl = 0;
	calCl = dataManager::instance()->getData("CsIClusterList",calCl);
    Float_t etot = calCl->Cluster(0)->energySum();

	unsigned int i = 0;
	for (i = 0; i < nLogs; i++) {
		CalRecLog* recLog = recLogs->Log(i);
		Float_t x = recLog->position().x();
		Float_t y = recLog->position().y();
		Float_t idEvent  = ide;
		Float_t id       = recLog->logID(); 
		Float_t eneNeg   = recLog->negEnergy();
		Float_t enePos   = recLog->posEnergy();
		Float_t ene      = recLog->energy();
		((TNtuple*) getFile()->Get("ntRecLogsEne"))->
			Fill(idEvent,id,eneNeg,enePos,ene,etot,nStrips,x,y);
	}

	for (i = 0; i < nLogs; i++) {
		CalRecLog* recLog = recLogs->Log(i);
		Float_t idEvent  = ide;
		Float_t id       = recLog->logID(); 
		Float_t ePosLEX   = recLog->posEnergy(CalBase::LEX);
		Float_t eNegLEX   = recLog->negEnergy(CalBase::LEX);
		Float_t ePosLE   = recLog->posEnergy(CalBase::LE);
		Float_t eNegLE   = recLog->negEnergy(CalBase::LE);
		Float_t ePosHEX   = recLog->posEnergy(CalBase::HEX);
		Float_t eNegHEX   = recLog->negEnergy(CalBase::HEX);
		Float_t ePosHE   = recLog->posEnergy(CalBase::HE);
		Float_t eNegHE   = recLog->negEnergy(CalBase::HE);
		((TNtuple*) getFile()->Get("ntRecLogsRan"))
			->Fill(idEvent,id,etot,nStrips,
			ePosLEX,ePosLE,ePosHEX,ePosHE,
			eNegLEX,eNegLE,eNegHEX,eNegHE);
	}

	for (i = 0; i < nLogs; i++) {
		CalRecLog* recLog = recLogs->Log(i);
		Float_t idEvent  = ide;
		Float_t id       = recLog->logID(); 
				Float_t ADCPosLEX = recLog->ADC(CalBase::POS,CalBase::LEX);
				Float_t ADCPosLE  = recLog->ADC(CalBase::POS,CalBase::LE);
				Float_t ADCPosHEX = recLog->ADC(CalBase::POS,CalBase::HEX);
				Float_t ADCPosHE  = recLog->ADC(CalBase::POS,CalBase::HE);
				Float_t ADCNegLEX = recLog->ADC(CalBase::NEG,CalBase::LEX);
				Float_t ADCNegLE  = recLog->ADC(CalBase::NEG,CalBase::LE);
				Float_t ADCNegHEX = recLog->ADC(CalBase::NEG,CalBase::HEX);
				Float_t ADCNegHE  = recLog->ADC(CalBase::NEG,CalBase::HE);
				((TNtuple*)getFile()->Get("ntRecLogsADC"))->
					Fill(idEvent,id,etot,nStrips,
					ADCPosLEX,ADCPosLE,ADCPosHEX,ADCPosHE,
					ADCNegLEX,ADCNegLE,ADCNegHEX,ADCNegHE);
			
	}

}
