#include "RootTree/SiHistoFolder.h"

#include "RootTree/rTreeRawServer.h"

#include "Event/dataManager.h"

#include "TFile.h"
#include "TH1.h"
#include "TH2.h"
#include "TNtuple.h"
#include "TROOT.h"

#include "TMath.h"
#include "TObject.h"
#include "TObjArray.h"
#include "TMap.h"

//###############################
void SiHistoFolder::define() 
//###############################
{	
	if (defined()) return;

	m_SiRecObjs  = dataManager::instance()->getData("SiRecObjs",m_SiRecObjs);
	m_SiLayers   = dataManager::instance()->getData("SiLayers",m_SiLayers);
	m_SiClusters = dataManager::instance()->getData("SiClusters",m_SiClusters);

	ntupleDefine(m_SiRecObjs);
	histosDefine(m_SiLayers);
	histosDefine(m_SiRecObjs);

	setDefine(true);
}

//###############################
void SiHistoFolder::fill() 
//###############################
{	
	if (!defined()) define();

	ntupleFill(m_SiRecObjs);
	histosFill(m_SiLayers);
	histosFill(m_SiRecObjs);
}

//------------ define -----------
//###############################
void SiHistoFolder::histosDefine(SiLayers* s)
//###############################
{
	getFile()->cd();
	TH2F* htBeamX = new TH2F("htBeamX", "Tracker hits - x planes", 100, 0, 2048, 16, 0, 16);
	TH2F* htBeamY = new TH2F("htBeamY", "Tracker hits - y planes", 100, 0, 2048, 16, 0, 16);
	TH2F* htHitsX = new TH2F("htHitsX", "num hits - x planes", 64, 0, 64, 16, 0, 16);
	TH2F* htHitsY = new TH2F("htHitsY", "num hits - y planes", 64, 0, 64, 16, 0, 16);
	TH1F* htNhits  = new TH1F("htNhits",  "num hits - tracker ", 100, 0, 100);
}
//###############################
void SiHistoFolder::histosDefine(SiRecObjs* s) 
//###############################
{	
	getFile()->cd();
	TH1F* htNtracks  = new TH1F("htNtracks","num tracks", 16, 0, 16);
	TH1F* htNgammas  = new TH1F("htNgammas","num gammas", 16, 0, 16);
	TH1F* htZvertex  = new TH1F("htZvertex","gamma - Z vertex",50,-10.,40.);
}

//###############################
void SiHistoFolder::ntupleDefine(SiRecObjs* s) 
//###############################
{
	getFile()->cd();
	char* varname = "IdEvent:Id:IdPlane:Nhits:Quality:ChiSq:InEne:X:Y:Z:Xdir:Ydir:Zdir";
	TNtuple* ntTrkRecGamma  = new TNtuple("ntTrkRecGamma", "trkRecGamma ntuple",varname);

	// char* varname = "IdEvent:Id:IdPlane:Nhits:Quality:ChiSq:InEne:X:Y:Z:Xdir:Ydir:Zdir";
	TNtuple* ntTrkRecParticle = new TNtuple("ntTrkRecParticle", "trkRecParticles ntuple",varname);
}
//------------ fill --------------
//###############################################
void SiHistoFolder::histosFill(SiLayers* siLayers)
//###############################################
{
	getFile()->cd();
	Float_t nStrips = siLayers->numStrips();
	((TH1F*) getFile()->Get("htNhits"))->Fill(nStrips); 

	int nLayers = siLayers->num();
	unsigned int i = 0;
	for (i = 0; i < nLayers; i++) {
		
		SiLayer* siLayer = siLayers->Layer(i);
		
		int view = siLayer->view(); // x=0 y=1
		char* name1 = "htHitsX";
		char* name2 = "htBeamX";
		if (view == 1) {
			name1 = "htHitsY";
			name2 = "htBeamY";
		}
		Float_t  idlayer = siLayer->layer();
		nStrips = siLayer->nstrips();
		((TH2F*) getFile()->Get(name1))->Fill(nStrips,idlayer);
		unsigned int j = 0;
		for (j = 0; j < nStrips ; j++) {
			Float_t is = siLayer->idstrip(j);
			((TH2F*) getFile()->Get(name2))->Fill(is,idlayer);
		}

	}
}
//###############################################
void SiHistoFolder::histosFill(SiRecObjs* siRec)
//###############################################
{
	getFile()->cd();
	Float_t nTracks = siRec->numParticles();
	((TH1F*) getFile()->Get("htNtracks"))->Fill(nTracks); 

	Float_t nGammas = siRec->numGammas();
	((TH1F*) getFile()->Get("htNgammas"))->Fill(nGammas); 

	int nGamma = siRec->numGammas();
	unsigned int i = 0;
	for (i = 0; i < nGamma; i++) {
		GFgamma* gamma = siRec->Gamma(i);
		((TH1F*) getFile()->Get("htZvertex"))->Fill(gamma->vertex().z()); 
	}
}
//###############################################
void SiHistoFolder::ntupleFill(SiRecObjs* siRec)
//###############################################
{
	getFile()->cd();
	Float_t ide = dataManager::instance()->IO()->treeIn()->eventNumber();
	unsigned int ngammas = siRec->numGammas();

	unsigned int i = 0;
	for (i = 0; i < ngammas; i++) {
		GFgamma* gamma = siRec->Gamma(i);
		Float_t idEvent=ide;
		Float_t id = i;
		Float_t idPlane = gamma->firstLayer();
		Float_t nHits = gamma->nhits();
		Float_t Quality = gamma->Q();
		Float_t ChiSq = 0;
		Float_t inEne = gamma->inputEnergy();
		Float_t X = gamma->vertex().x();
		Float_t Y = gamma->vertex().y();
		Float_t Z = gamma->vertex().z();
		Float_t Xdir = gamma->direction().x();
		Float_t Ydir = gamma->direction().y();
		Float_t Zdir = gamma->direction().z();
		((TNtuple*) getFile()->Get("ntTrkRecGamma"))->Fill(
			idEvent,id,idPlane,nHits,Quality,ChiSq,inEne,
			X,Y,Z,Xdir,Ydir,Zdir);
	}


	unsigned int npar = siRec->numParticles();
	for (i = 0; i < npar; i++) {
		GFparticle* par = siRec->Particle(i);
		Float_t idEvent=ide;
		Float_t id = i;
		Float_t idPlane = par->firstLayer();
		Float_t nHits =   par->nhits();
		Float_t Quality = par->Q();
		Float_t ChiSq   = par->getXGFtrack()->chiSquare()+par->getYGFtrack()->chiSquare();
		Float_t inEne = par->inputEnergy();
		Float_t X = par->vertex().x();
		Float_t Y = par->vertex().y();
		Float_t Z = par->vertex().z();
		Float_t Xdir = par->direction().x();
		Float_t Ydir = par->direction().y();
		Float_t Zdir = par->direction().z();
		((TNtuple*) getFile()->Get("ntTrkRecParticle"))->Fill(
			idEvent,id,idPlane,nHits,Quality,ChiSq,inEne,
			X,Y,Z,Xdir,Ydir,Zdir);
	}
}