#include "RootTree/TowerHistoFolder.h"

#include "instrument/CalBase.h"
#include "Event/dataManager.h"
#include "RootTree/rTreeRawServer.h"
#include "generation/SiLayers.h"
#include "reconstruction/SiRecObjs.h"
#include "reconstruction/CsIClusters.h"

#include "RootTree/rTreeRawServer.h"

#include "TFile.h"
#include "TH1.h"
#include "TH2.h"
#include "TNtuple.h"
#include "TROOT.h"

#include "TMath.h"
#include "TObject.h"
#include "TObjArray.h"
#include "TMap.h"

//###############################
void TowerHistoFolder::define() 
//###############################
{	
	if (defined()) return;
	
	m_tkr = dataManager::instance()->getData("SiRecObjs",m_tkr);
	m_cal = dataManager::instance()->getData("CsIClusterList",m_cal);
	m_tagger = dataManager::instance()->getData("BeamTagger",m_tagger);

	ntupleDefine(m_tkr,m_cal,m_tagger);

	setDefine(true);
}

//###############################
void TowerHistoFolder::fill() 
//###############################
{	
	if (!defined()) define();
	
	ntupleFill(m_tkr,m_cal,m_tagger);
}
//###############################
void TowerHistoFolder::ntupleDefine(SiRecObjs* s, CsIClusterList* lg, BeamTagger* tg) 
//###############################
{
	getFile()->cd();

	char* varname = "IdEvent:TaggerEne:CalEne:GammaXdir:GammaYdir:GammaZdir:FirstLayer:VertexX:VertexY:VertexZ";
	TNtuple* ntRecGamma = new TNtuple("ntRecGamma", "Reconstructed gamma-ray ntuple", varname);

	// ascii output (temporary)
//	std::ostream out = messageManager::instance()->out();
//	out << "GAMMARAY\tIdEvent\tTaggerEne\tCalEne\tGammaXdir\tGammaYdir\tGammaZdir\tFirstLayer\tVertexX\tVertexY\tVertexZ\n";
}
//###############################################
void TowerHistoFolder::ntupleFill(SiRecObjs* siRec, CsIClusterList* cal, BeamTagger* tagger)
//###############################################
{
	getFile()->cd();

	// output "clean" single-gamma event only
	//if (siRec->numGammas() < 1) return;
	//if (!tagger->clean()) return;
	//if (tagger->nParticle() != 1) return;
	//if (tagger->particleID(0) != 2) return;

	// event ID
	Float_t ide = dataManager::instance()->IO()->treeIn()->eventNumber();

	// tagger energy
	Float_t tagEne = 0.;
	Float_t clean = 0;
	Float_t npart = 0;
	if (tagger->nParticle() >=1) { 
		tagEne = tagger->momentum(0);
		clean = tagger->clean();
		npart = tagger->nParticle();
	}

	// calorimeter energy
	Float_t calEne = 0.;
	for (int i=0; i<cal->num(); i++) {
		calEne += (Float_t)cal->Cluster(i)->energySum();
	}

	// reconstructed gamma-ray
	Float_t Xdir  = 0.;
	Float_t Ydir  = 0.;
	Float_t Zdir  = 0.;
	Float_t Layer = 0.;
	Float_t Xvtx  = 0.;
	Float_t Yvtx  = 0.;
	Float_t Zvtx  = 0.;
	if (siRec->numGammas() == 1) {
		GFgamma* gamma = siRec->Gamma(0);
		Xdir  = gamma->direction().x();
		Ydir  = gamma->direction().y();
		Zdir  = gamma->direction().z();
		Layer = gamma->firstLayer();
		Xvtx  = gamma->vertex().x();
		Yvtx  = gamma->vertex().y();
		Zvtx  = gamma->vertex().z();
	}
	((TNtuple*) getFile()->Get("ntRecGamma"))->Fill(ide, tagEne, calEne, Xdir, Ydir, Zdir,
		Layer, Xvtx, Yvtx, Zvtx);

}
