#include "rTreeReconServer.h"

#include "RootTree/converterImport.h"

#include "Recon.h"

#include "Event/rTreeBase.h"
#include "Event/converterServer.h"

#include "RootTree/TkrSiCluster.h"
#include "RootTree/TkrTrack.h"
#include "RootTree/TkrRecon.h"
#include "RootTree/Recon.h"

#include "RootTree/TkrTrack.h"
#include "RootTree/TkrLocator.h"
#include "RootTree/TkrGamma.h"
#include "RootTree/TkrHit.h"

#include "TFile.h"
#include "TTree.h"

#include "reconstruction/SiRecObjs.h"
#include "reconstruction/SiClusters.h"
#include "reconstruction/CalRecLogs.h"
#include "reconstruction/CsIClusters.h"
#include "RootTree/CalLogEne.h"
#include "RootTree/CalCluster.h"
#include "RootTree/CalRecon.h"

#include "analysis/tbHeader.h"
#include "RootTree/ReconHeader.h"

//#######################################
void rTreeReconServer::defineConverters()
//#######################################
{
	/*! add the converter from CsIClusterList */
	addConverter("CsIClusterList",  
		new converterImport<rTreeReconServer,CsIClusterList>(this,"CsIClusterList") );
	
	/*! add the converter from CalRecLogs */
	addConverter("CalRecLogs", 
		new converterImport<rTreeReconServer,CalRecLogs>(this,"CalRecLogs") );
	
	/*! add the converter from tbHeader */
	addConverter("tbHeader",	 
		new converterImport<rTreeReconServer,tbHeader>(this,"tbHeader") );
	
	/*! add the converter from SiClusters */
	addConverter("SiClusters", 
		new converterImport<rTreeReconServer,SiClusters>(this,"SiClusters") );
	
	/*! add the converter from tbHeader */
	addConverter("SiRecObjs",
		new converterImport<rTreeReconServer,SiRecObjs>(this,"SiRecObjs") );

	/*! create the load and fill algorithms with the converters */
	makeAlgorithms();
}
//--------------------------------------------------------------


void rTreeReconServer::save(CsIClusterList *ccl)
{
	int nc = ccl->num();
	TObjArray* clusArr = core()->getCalRecon()->getCalClusters();
	for (int ic = 0; ic<nc; ic++){
		CsICluster* csiclus = ccl->Cluster(ic);
		CalCluster* calclus = converter(csiclus);
		calclus->setId(ic);
		clusArr->Add(calclus);
	}
}

CalCluster* rTreeReconServer::converter(const CsICluster *cc)
{
	CalCluster* calclus = new CalCluster();	
	calclus->setEsum(cc->energySum());
	calclus->setEcorr(cc->energyCorrected());
	Point pos = cc->position();
	calclus->setPosition(pos.x(),pos.y(),pos.z());
	Vector dir = cc->direction();
	calclus->setDirection(dir.x(), dir.y(),dir.z());
	calclus->setCsiAlpha(cc->getCsiAlpha());
	calclus->setCsiLambda(cc->getCsiLambda());
	calclus->setCsiStart(cc->getCsiStart());
	calclus->setFitEnergy(cc->getFitEnergy());
	calclus->setProfChisq(cc->getProfChisq());
	for(int i=0;i<8;i++){
		calclus->setEneLayer(i,cc->getEneLayer(i));
		const Vector& posl = cc->getPosLayer(i);
		calclus->setXLayer(i,posl.x());
		calclus->setYLayer(i,posl.y());
	}
    // for the moment we have only one cluster and I include
	// all hited logs into its log list
	// this should be modified when real clusterisation
	// will be added
	TObjArray* logArr = core()->getCalRecon()->getCalLogs();
	TObjArray* cluslogs = calclus->getCalLog();
	TObjArrayIter it(logArr);
	TObject* log;
	while(log = it.Next())cluslogs->Add(log);
	
	return calclus;
}

void rTreeReconServer::save(CalRecLogs *crls)
{
	int nlogs = crls->num();
	TObjArray* logArr = core()->getCalRecon()->getCalLogs();
	
	for (int il=0; il<nlogs; il++){
		CalRecLog* crl = crls->Log(il);
		if(crl->posEnergy()>0 || crl->negEnergy()>0){
  			CalLogEne* cle = converter(crl);
			logArr->Add(cle);
		}
	}
}


CalLogEne* rTreeReconServer::converter(const CalRecLog* crl)
{
		CalLogEne* cle = new CalLogEne();
		cle->setePlus(crl->posEnergy());
		cle->seteMinus(crl->negEnergy());
		int tb_layer = crl->layer();
		int tb_col = crl->column();
		int tb_view = crl->view();
		int root_layer = 7-(2*tb_layer+tb_view);
		int root_col = (tb_view ? tb_col:9-tb_col);
		LogID* id = new LogID();
		id->setColumn(root_col);
		id->setLayer(root_layer);
		cle->setId(id);
		return cle;	
}

//-------------- Header -------------

//#######################################
void rTreeReconServer::save(tbHeader* header)
//#######################################
{
	core()->getReconFlags()->setEventID(header->getEvent());
	core()->getReconFlags()->setTkrFlag(header->getInfoTkr());
}
//-------------- Si Clusters -------------

//#######################################
void rTreeReconServer::save(SiClusters* sicl)
//#######################################
{
	int nhits = sicl->nHits();
	unsigned int ihit = 0;
    // pointer to the SiCluster TObjArray
   	TObjArray *clusterArr = core()->getTkrRecon()->getSiClusters();
    // loop over all hits 
	for (ihit = 0; ihit < nhits; ihit ++) {
		SiCluster*    scl = sicl->getHit(ihit);
		TkrSiCluster* rcl = converter(scl);
		clusterArr->Add(rcl);
	}
}
//#######################################
TkrSiCluster* rTreeReconServer::converter(const SiCluster* scl)
//#######################################
{
	TkrSiCluster* rcl = new TkrSiCluster();
	rcl->setCenterStrip(scl->strip());
	rcl->setId(scl->id());
	rcl->setLayer(scl->plane());
	rcl->setNumStrips(scl->size());
	Point p = scl->position();
	Float_t x = (scl->v() == 0? p.x():p.y());
	rcl->setPosition(x);
	TkrSiCluster::TKRAxes axis;
	axis = (scl->v()==0? TkrSiCluster::X : TkrSiCluster::Y);
	rcl->setXY(axis);
	rcl->setZPosition(p.z());
	return rcl;
}
//---------------- Tracks --------------------
//#######################################
void rTreeReconServer::save(SiRecObjs* sirec)
//#######################################
{	
	importGamma(sirec);
	importTracks(sirec);
}
//#######################################
void rTreeReconServer::importGamma(SiRecObjs* sirec)
//#######################################
{

	int ngamma = sirec->numGammas();
	if (ngamma !=1 ) return;

    // pointer to TrkGamma
   	TkrGamma* rgamma = new TkrGamma(1);
	core()->getTkrRecon()->setGamma(rgamma);
		
	GFgamma* gamma = sirec->Gamma(0);
	TkrLocator* rloc = converter(gamma);
	((TObjArray*) rgamma->getLocator())->Add(rloc);
	TObjArray* rgammaTracks = rgamma->getTracks();

	TObjArray* rtracks = core()->getTkrRecon()->getTracks();

	m_idtrack = 0;

	const GFtrack* xbest = gamma->getBest(SiCluster::X);
	TkrTrack* rbxtrack = converter(xbest,0,true);
	rbxtrack->setId((UShort_t) m_idtrack++);
	rtracks->Add(rbxtrack);
	rgammaTracks->Add(rbxtrack);

	const GFtrack* ybest = gamma->getBest(SiCluster::Y);
	TkrTrack* rbytrack = converter(ybest,0,true);
	rbytrack->setId( (UShort_t) m_idtrack++);
	rtracks->Add(rbytrack);
	rgammaTracks->Add(rbytrack);

	const GFtrack* xpair = gamma->getPair(SiCluster::X);
	if (!xpair->empty()) {
		TkrTrack* rpxtrack = converter(xpair,1,true);
		rpxtrack->setId( (UShort_t) m_idtrack++);
		rtracks->Add(rpxtrack);
		rgammaTracks->Add(rpxtrack);
	}

	const GFtrack* ypair = gamma->getPair(SiCluster::Y);
	if (!ypair->empty()) {
		TkrTrack* rpytrack = converter(ypair,1,true);
		rpytrack->setId( (UShort_t) m_idtrack++);
		rtracks->Add(rpytrack);
		rgammaTracks->Add(rpytrack);
	}

	if (rtracks->GetEntries() <=0) {
		int badbad = 1;
	}
}

//#######################################
void rTreeReconServer::importTracks(SiRecObjs* sirec)
//#######################################
{
	int nparticles = sirec->numParticles();
	if (nparticles <=0) return;

	TObjArray* rTracks = core()->getTkrRecon()->getTracks();
	for (int iparticle = 0; iparticle < nparticles; iparticle++) {
		GFparticle* particle = sirec->Particle(iparticle);
		const GFtrack* xtrack = particle->getXGFtrack();
		TkrTrack* rxtrack = converter(xtrack,iparticle);
		rxtrack->setId( (UShort_t) m_idtrack++);
		rTracks->Add(rxtrack);
		const GFtrack* ytrack = particle->getYGFtrack();
		TkrTrack* rytrack = converter(ytrack,iparticle);
		rytrack->setId( (UShort_t) m_idtrack++);
		rTracks->Add(rytrack);
	}
}

//#######################################
void rTreeReconServer::importHits(const GFtrack* track, TkrTrack* rtrack)
//#######################################
{
	int nhits = track->nhits();
	if (nhits <=2) return;

	SiCluster::view a = track->getAxis();
	TObjArray* rHits = rtrack->getHits();
	for (int ihit = 0; ihit < nhits; ihit++) {
		const KalPlane& kplane = track->kplanelist[ihit];
		TkrHit* rhit = converter(kplane,a);
		rHits->Add(rhit);
	}
	if (rHits->GetEntries() <=0 ) {
		int badbad = 1;
	}
}
//------------------ converters ---------------
//#######################################
TkrLocator* rTreeReconServer::converter(const GFdata* gf)
//#######################################
{

	TkrLocator* rloc = new TkrLocator(1);
	Float_t x = gf->vertex().x();
	Float_t y = gf->vertex().y();
	Float_t z = gf->vertex().z();
	rloc->setPosition(new TVector3(x,y,z));
	
	Float_t tx = gf->direction().x();
	Float_t ty = gf->direction().y();
	Float_t tz = gf->direction().z();
	Float_t slopex  = (tz != 0. ? slopex = tx/tz : 0);
	Float_t slopey  = (tz != 0. ? slopey = ty/tz : 0);
	rloc->setDirection(new TVector2(slopex,slopey));
	return rloc;
}
//#######################################
TkrTrack* rTreeReconServer::converter(const GFtrack* track, int id, bool mother)
//#######################################
{
	TkrTrack* rtrack = new TkrTrack(1);
	
	// set the locator at the vertex
	TkrLocator* rloc = new TkrLocator(1);
	rloc = converter(track);
	rtrack->getLocator()->Add(rloc);

	// set the other information
	Float_t chi2 = track->chiSquare();
	rtrack->setChi2(chi2);

	Float_t eneDet = track->RCenergy();
	rtrack->setEnergyDet(eneDet);

	Float_t eneIn  = track->inputEnergy();
	rtrack->setEnergyInput(eneIn);

	UInt_t fstLayer = track->firstLayer();
	rtrack->setFirstLayer(fstLayer);
	
	rtrack->setId(id);

	if (mother) rtrack->setMother(TkrTrack::GAMMA);
	else rtrack->setMother(TkrTrack::NONE);

	Float_t quality = track->Q();
	rtrack->setQuality(quality);
	
	Float_t residual = 0.;
	rtrack->setResidual(residual);

	rtrack->setTrackType(TkrTrack::ELECTRON);

	importHits(track, rtrack);

	return rtrack;
}
//#######################################
TkrHit* rTreeReconServer::converter(const KalPlane& kp, SiCluster::view a)
//#######################################
{
	TkrHit* rHit = new TkrHit();

	Float_t chi2 = kp.getDeltaChiSq(KalHit::SMOOTH);
	rHit->setChi2(chi2);
	
	Float_t xmea = kp.getHit(KalHit::MEAS).getPar().getPosition();
	Float_t xfit = kp.getHit(KalHit::SMOOTH).getPar().getPosition();
	Float_t res = xfit-xmea;
	rHit->setResidual(res);

	TkrLocator* rloc = converter(kp,a,1);
	rHit->setLocator(rloc);

	int ihit = kp.getIDHit();
	TkrSiCluster* rcl = core()->getTkrRecon()->getSiCluster(ihit);
	rHit->setSiCluster(rcl);

	return rHit;
}

//#######################################
TkrLocator* rTreeReconServer::converter(const KalPlane& kp, SiCluster::view a, int idummy)
//#######################################
{
	TkrLocator* rloc = new TkrLocator(1);
	Float_t x = 0;
	Float_t y = 0;
	Float_t xp = kp.getHit(KalHit::SMOOTH).getPar().getPosition();
	if (a == SiCluster::X) x = xp;
	else y = xp;
	Float_t z = kp.getZPlane();
	rloc->setPosition(new TVector3(x,y,z));
	
	Float_t slopex = 0.;
	Float_t slopey = 0.;
	Float_t slopep = kp.getHit(KalHit::SMOOTH).getPar().getSlope();
	if (a == SiCluster::X) slopex = slopep;
	else slopey = slopep;
	rloc->setDirection(new TVector2(slopex,slopey));

	Float_t ex = sqrt(kp.getHit(KalHit::SMOOTH).getCov().getsiga());
	Float_t es = sqrt(kp.getHit(KalHit::SMOOTH).getCov().getsigb());
	Float_t ec = kp.getHit(KalHit::SMOOTH).getCov().getcovab();

	if (a == SiCluster::X) rloc->setXmatrix(ex,es,ec);
	else rloc->setYmatrix(ex,es,ec);

	return rloc;
}
