
#ifndef rTreeReconServer_h
#define rTreeReconServer_h

#include "Recon.h"

#include "Event/rTreeBase.h"
#include "Event/converterServer.h"

#include "reconstruction/SiRecObjs.h"
#include "reconstruction/SiClusters.h"
#include "reconstruction/CalRecLogs.h"
#include "reconstruction/CsIClusters.h"
#include "analysis/tbHeader.h"

class CalLogEne;
class CalCluster;

class TkrTrack;
class TkrSiCluster;
class TkrLocator;
class TkrHit;
class TkrGamma;

//############################################
class rTreeReconServer : public rTreeBase<Recon>, public converterServer
//#############################################
{	
public:

	//! constructor: create the converters
	/*! the constructor of converterServer will call defineConverters*/
	rTreeReconServer():rTreeBase<Recon>(new Recon(),"Recon") {defineConverters();}

	//! save the tracker hight level reconstructed objects from transient to persistent storage
	void save(SiRecObjs* sirec);
	//! save the tracker medium level reconstructed objects from transient to persistent storage 
	void save(SiClusters* sicl);
	//! save the CalRecLogs 
	void save(CalRecLogs* crls);
	//! save the CsIClusterList
	void save(CsIClusterList* ccl);
	//! save the header information
	void save(tbHeader* header);

	//! null load the tracker hight level reconstructed objects from transient to persistent storage
	void load(SiRecObjs* sirec) {};
	//! null load the tracker medium level reconstructed objects from transient to persistent storage 
	void load(SiClusters* sicl) {};
	//! null load the CalRecLogs 
	void load(CalRecLogs* crls) {};
	//! null load the CsIClusterList
	void load(CsIClusterList* ccl) {};
	//! null load the header information
	void load(tbHeader* header) {};

protected:

	//! The User should define in this methods the converters from Recon->Transient
	/*! The system will not know the persistent data and will interact only
	with the algorithms created for the converters that the User declare here */
	virtual void defineConverters();

private:

	CalLogEne*    converter(const CalRecLog* crl);
	CalCluster*   converter(const CsICluster* cc);

	//! converter for the reconstructed tracks
	/*! A GFtrack (3D) tracks is divided into 2 "TkrTrack" (X and Y) projectios.
	The ID of a "TkrTrack" associated the X-Y projection of the same 3D "GFtrack"*/
	TkrTrack*     converter(const GFtrack* track, int id, bool mother = false);
	//! converter to locate an elemento of the track (i.e. vertex)
	/*! A "TkrLocator" contains information about the position, direction and errors of
	a reconstructed track in a given position*/
	TkrLocator*   converter(const GFdata* gf);
	//! convertor from transiend to persistend data of the same class "SiCluster" (a simple copy)
	TkrSiCluster* converter(const SiCluster* scl);

	//! convertor (constructor) of a "TkrHit" object from a "KalPlane" object
	/*! A "TkrHit" contains information of the "TkrLocator" of a given hit, its 
	chi2, residual, and the "SiCluster" which is associated with" */
	TkrHit*       converter(const KalPlane& kp, SiCluster::view a);
	//! converto from a "KalPlane" object to a "TkrLocator"
	/*! The "KalPlane" objects contain information for the SiClusters and the fitted hit
	in a given position of the track*/
	TkrLocator*   converter(const KalPlane& kp, SiCluster::view a, int idummy);

private:

	//! import the tracker reconstructed gamma from "SiRecObjs" to persistent
	void importGamma(SiRecObjs* sirec);
	//! import the reconstructed tracks from "SiRecObjs" into persistent 
	void importTracks(SiRecObjs* sirec);
	//! import the hits of a given track from transient ("KalPlane") to persistent ("TkrHit")
	void importHits(const GFtrack* track, TkrTrack* rtrack);

private:

	//! counter of the number of saved track per event
	int m_idtrack;

};
#endif


