
#include "centellaGUI/graphicsManager.h"
#include "centellaGUI/AppliGUI.h"
#include "Event/dataManager.h"
#include "reconstruction/SiClusters.h"
#include "reconstruction/SiRecObjs.h"
#include "reconstruction/CalRecLogs.h"
#include "Event/processManager.h"
#include "Event/selectionManager.h"
#include "Event/messageManager.h"

double screensize=-40.; 

graphicsManager* graphicsManager::m_instance = 0;

//###########################################
graphicsManager::graphicsManager()
//###########################################
{
	// create windoes
	m_gui     = GUI::createGUI("centella","centella"); 
	m_display = new DisplayControl(*Menu::instance(), 
		m_gui->graphicsWindow(screensize));
	m_print   = new PrintControl(m_gui->textWindow("output"));

	m_instance = this;
}
//###########################################
graphicsManager::~graphicsManager()
//###########################################
{
	delete m_gui;
//	delete m_menu;
	delete m_display;
	delete m_print;
}

//###########################################
void graphicsManager::define()
//###########################################
{

	// Add commands 
	m_gui->setTitle("Centella");
	m_gui->addToMenu("One",new One);
	m_gui->addToMenu("Recon",new Rec);
	m_gui->addToMenu("Go",new Go);

	// add display objects
//	m_display->add(new Axis(20.),"axis",true);
	m_display->add(new ArveDisplayer<trackerDetGeo> 
		(dataManager::instance()->geo()->tracker()),"tracker",true);
	m_display->add(new ArveDisplayer<calorimeterDetGeo> 
		(dataManager::instance()->geo()->cal()),"calorimeter",true);
	SiClusters* sicl = 0;
	sicl = dataManager::instance()->getData("SiClusters",sicl);
	m_display->add(new ArveDisplayer<SiClusters>(sicl),"clusters",true);
	SiRecObjs* sirec = 0;
	sirec = dataManager::instance()->getData("SiRecObjs",sirec);
	m_display->add(new ArveDisplayer<SiRecObjs>(sirec),"tracks",true);
	CalRecLogs* calLogs = 0;
	calLogs = dataManager::instance()->getData("CalRecLogs",calLogs);
	m_display->add(new ArveDisplayer<CalRecLogs>(calLogs),"recLogs",true);

	m_display->update();
}

//###########################################
void graphicsManager::run()
//###########################################
{
	m_gui->run();
}
//###########################################
void graphicsManager::update()
//###########################################
{
	// display clear and update
	m_display->clear();
	m_display->update();
	m_display->redisplay();
	m_print->printAll();
}
//------------- comands --------------------

//##########################################
void One::execute()
//##########################################
{
//	messageManager::instance()->open("centellita.out");
	algorithmVI* runEvent = processManager::instance()->getAlgorithm("runEventAlg");
	runEvent->run();
	graphicsManager::instance()->update();
//	messageManager::instance()->close();
	GUI::instance()->inform("Loaded new Event");
}
//##########################################
void Rec::execute()
//##########################################
{
	algorithmVI* processEvent = processManager::instance()->getAlgorithm("runEvent");
	processEvent->run();
	graphicsManager::instance()->update();
	GUI::instance()->inform("Recon Event");
}

//##########################################
void Go::execute()
//##########################################
{
	algorithmVI* runEvent = processManager::instance()->getAlgorithm("runEventAlg");
	cutVI* selRead = selectionManager::instance()->getCut("selRead");
	bool end = false;
	while (!end) {
		if (selRead->apply()) end = true;
		runEvent->run();
		graphicsManager::instance()->update();
	}
	// information
	int ievt = dataManager::instance()->IO()->eventNumber();
	messageManager::instance()->message(" Event ",ievt);
//	dataManager::instance()->evt()->writeOutServer();
	GUI::instance()->inform("Go ended");
}
