// Project	: General Utilities Library
// Subproject	: Log
// File		: Warning.cxx
// Author	: Patrick Hendriks
// Version	: 1.0
// Created	: 02-07-1997 (1.0)
// Last revision: 21-04-1998 (1.0)


#include "Warning.h"

namespace util {


// *************************************************************************
// Global var.	: warning
// Type		: Warning
// Description	:
//	Common warning object to be used by all clients.

Warning
warning = Warning();


// *************************************************************************
// Class	: Warning
// Static var.	: ms_new_warning
// Access	: Protected, static
// Description	:
//	True when the next output begins a new warning.

bool
Warning::ms_new_warning = true;


// *************************************************************************
// Class	: Warning
// Method	: Constructor
// Access	: Public
// Description	:
//	Create a new warning object.

Warning::Warning() :
    Messenger("WARNING: ")
{
}


// *************************************************************************
// Class	: Warning
// Method	: stream
// Access	: Public
// Description	:
//	If m_new_warning is set, advance to the next line before calling
//	the same method in Messenger.

std::ostream&
Warning::stream()
{
    if (ms_new_warning) {
	ms_new_warning = false;
	if (!ms_new_line) {
	    Messenger::operator<<(std::endl);
	}
    }
    return Messenger::stream();
}


// *************************************************************************
// Class	: Warning
// Method	: operator<<
// Access	: Public
// Description	:
//	Set m_new_warning to true and call the corresponding method in
//	Messenger.

Messenger&
Warning::operator<<(std::ostream& (*p_func)(std::ostream&))
{
    ms_new_warning = true;
    return Messenger::operator<<(p_func);
}


}	// End of the util namespace
