// $Heading: ArveModel.h $
//   Author: Toby Burnett
//  Project: Arve graphics
//
// Templated class that defines a model assuming arve graphics, in the
// context of the general Atlas Event Display model
//
// Usage:
//     PlottableModel* APlottable::model()const
//     {   return new ArveModel<APlottable,ARep>(this); }


#ifndef ARVEMODEL_H
#define ARVEMODEL_H

#include "arve/config.h"
#include "display/PlottableModel.h"
#include "display/AbstractScene.h"
#include <assert.h>

template< class P, class R>
class ArveModel : public PlottableModel {
public:
    ArveModel(const P* plottable): m_plottable(*plottable){}
    ArveModel(const P& plottable): m_plottable(plottable){}
    // constructor:: just set data member

    // Implicit copy constructor
    // Implicit destructor
    // Implicit assignment operator

    virtual PlottableRep* 
    representation(const AbstractScene& aScene)const;
	
private:
    const P& m_plottable;
    // pointer to the plottable object 
};
//-------------inlines----------------------------
template<class P, class R>
inline PlottableRep*
ArveModel<P,R>::representation(const AbstractScene& aScene)const
{
	assert( aScene.type()==AbstractScene::ARVE );
	return new R(m_plottable);
}
#endif //ARVEMODEL
