// $Header: /cvs/tb_recon/graphics/ArvePlottableRep.h,v 1.1.1.1 2000/04/03 21:03:56 wilko Exp $
//  Author: Toby Burnett
// Project: Arve graphical display
//
// Defines a Plottable Representation that is tuned to the Arve environment
// Almost all of the functionallity is derived from DisplayList. The most important are:
//
// 	void move_to(const GraphicsVector&);
//	// move current point to location
//
//	void line_to(const GraphicsVector&);
//	// draw a line to the specified point
//
//	void drawText(const std::string&);
//	// draw the text at the current point
//
//	void markerAt(const GraphicsVector&);
//	// draw a marker at the given point
//
// where GraphicsVector is a 3-vector with automatic conversion from Hep3Vector, so client
// can call with Hep3Vector, Point, or Vector. (Although only Point makes sense!)
//
// This is an abstract class: an subclass must implement the method
//     virtual void update();
// which must generate the list of calls.  Another important virtual method is
//     virtual void clear();
// The default behaviour is to eliminate the display, which may be bypassed by defining
// a dummy.
//

#ifndef ARVEPLOTTABLEREP_H
#define ARVEPLOTTABLEREP_H

#include "graphics/DisplayList.h"
#include "graphics/GraphicsVector.h"

class ArvePlottableRep :  public DisplayList
{
public:	
	// Implicit constructor
	// Implicit copy constructor
	// Implicit assignment operator
	// Implicit destructor
    enum LineStyle {	  /* Line styles for get/setlinestyle */
	SOLID_LINE= 0, DOTTED_LINE = 1,  CENTER_LINE= 2,  DASHED_LINE = 3, USERBIT_LINE = 4};


    void set_color(const std::string& color_name);
    void setColor(const std::string& color_name);
    // set the color for subsequent operations
 
    void moveTo(const GraphicsVector&);
    void lineTo(const GraphicsVector&);
    // for backward compatibility -- inlined below

    virtual void clear();
    // remove the representation

    void append(ArvePlottableRep& other);
    void append(const ArvePlottableRep& other);
    // append another representation to this one (removes representation)

    ArvePlottableRep& nested();
    // return a representation that is nested within this one

    bool empty()const;
    // is it empty?

    struct ColorInfo { const char * name; unsigned r, g,b; };
    static ColorInfo pallete[];
    // color palette management
private:

    class Dummy;

};

//////////////// Inlines only to provide some backward compatibility/////////////
inline void ArvePlottableRep::setColor(const std::string& n){set_color(n);}
inline void ArvePlottableRep::moveTo(const GraphicsVector&v){move_to(v);}
inline void ArvePlottableRep::lineTo(const GraphicsVector&v){line_to(v);}
/////////////////////////////////////////////////////////////////////////////////
#endif //ARVEPLOTTABLEMODEL_H

