//     $Id: DisplayControl.h,v 1.3 2000/05/11 18:58:35 hernando Exp $
// Project: Arve display
//  Author: Toby Burnett
//
//

#ifndef DISPLAYCONTROL_H
#define DISPLAYCONTROL_H

#ifdef __GNUG__
#pragma interface
#endif

#include <string>

#include "control/Command.h"
#include "control/Menu.h"
class ArveScene;

class Draw2D;

#include "graphics/ArvePlottableRep.h"
#include "graphics/Arve2D.h"

class DisplayControl  {
// definition of the class DisplayControl--handle event display. It
// has a list of <<ArvePlottableRep>> objects which it manages, and sets up a "Display"
// menu.

public:

    DisplayControl(Menu& menu, SceneControl* control); 
    // constructor with reference to menu, scene controler

    ~DisplayControl();
    
    void add(ArvePlottableRep* arep, const std::string& name=std::string(""), int state=1);
    // Add an element to the display
    // It the name argument is present, create a  button for it under the "display"
    // pull-down menu. If state is 0 or 1 it will be a toggle, starting show/hide. 
    // Otherwise create separate show and hide buttons
    // if a submenu is 

    void add(Arve2D* arep, const std::string& name=std::string(""), int state=1);
    // add  element to 2-d display

    void setTitle(const std::string& title);
    // set the title bar of the associated window

    void setAxisSize(float size=1);

    const Hep3Vector& reference_point()const;
    // access to current reference point
    
    void postScript();
    // dump current view to a file for postscript (if viewport is a Projector)

    void vrml(std::ostream& out);
    // write out a VRML file

    void update();
    // ask scene to update

    void redisplay();
    // redisplay all objects 

    void clear();
    // clear display (send message to the scene)
    
    void finishSetup();
    // adds menu items after all user buttons

    ArveScene& scene();
    // access to the 3-d scene

    SubMenu& menu(); 
    // return the (sub)menu associated with the display

    void useMenu(SubMenu* s=0);
    // set a different submenu (which could be a sub-submenu of this Display menu)
    // if no argument, revert to the standard

private:
    void add_button( ArvePlottableRep* arep, const std::string& name, int state);
    // internal routine

    class HideIt; class ShowIt; // used for commands
    class VRML_command; 
    class PrintInstructions;
    class Axes;                 // nested class that creates axes display
    class ReferencePoint;       // nested class that will display the reference point
    
    SubMenu*	m_sub_menu;     // special sub menu we control
    SubMenu*    m_user_menu;    // user-settable menu

    ArveScene*	m_scene;   	// Scene (3d )
    ArveScene*  m_2d_scene;     // 2d scene
    SceneControl* m_control;	// Scend controller
    
    float 	m_axis_size;
    bool	m_running;	// used to keep track of startup

    Draw2D*     m_canvas;	// access to the 2-d canvas
};

inline SubMenu& DisplayControl::menu(){return *m_sub_menu;} 
inline ArveScene& DisplayControl::scene(){return *m_scene;}
#endif 

