//     $Id: ViewPort.cxx,v 1.1.1.1 2000/04/03 21:03:58 wilko Exp $
// Project: Atlas top-level prototype
//  Author: Toby Burnett
//
//

#include "ViewPort.h"
#include "CLHEP/Vector/ThreeVector.h"
#include "GraphicsVector.h"

float ViewPort::defaultDetail= 0.01f;

ViewPort::ViewPort(const Hep3Vector* _center, float sc,const float* _angles, const char* t)
: title(t)
, aspectRatio(1)
, scale(sc)
, center(_center)
, angles(_angles)


{
  reset();
}

void ViewPort::reset()  
{ 
   _detail = defaultDetail;
   zoomFactor =1; perspective=0;
 
   float identity[]={1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1};
   for(int i=0; i<16; i++){ R[i] = identity[i];}
   rotateEuler(angles);

   perspective = angles[3];

}

void
ViewPort::transformToWindow(const GraphicsVector& pt, float& winx,float& winy)
{
	float deltax = pt.x() - center->x();   //ugly, but efficient, I hope
	float deltay = pt.y() - center->y();
	float deltaz = pt.z() - center->z();
	float t = zoomFactor/scale;
	winx = (R[0]*deltax + R[4]*deltay + R[8]*deltaz)*t;
	winy = (R[1]*deltax + R[5]*deltay + R[9]*deltaz)*t*aspectRatio;

	if( perspective > 0.){
	   float winz = (R[2]*deltax + R[6]*deltay + R[10]*deltaz)*t;
	   float tt = 1.- winz*perspective;
	   if( tt > 1.e-3){
		winx /= tt; winy /= tt;
	    }
	}
}
 
Hep3Vector 
ViewPort::transformToWorld(float winx, float winy)
{
   return Hep3Vector
          ( (winx*R[0] + winy*R[1]) * scale/zoomFactor
          , (winx*R[4] + winy*R[5]) * scale/zoomFactor/aspectRatio
	  , (winx*R[8] + winy*R[9]) * scale/zoomFactor 
	  ); 
} 


float ViewPort::detail()const{  return _detail;}
void ViewPort::setDetail(float f){_detail=f;}
//----------------------------------------------------------------------
//                      Rotations--very efficient

void
ViewPort::rotateEuler(const float * angles)
{
        rotateZ(angles[0]);
        rotateY(angles[1]);
        rotateZ(angles[2]);
}

const static double d2r = M_PI/180;
static double c, s;

static void
rotate2(float& a, float &b)
{
  float t = c*a-s*b;
  b = s*a+c*b;
  a =t;
}  
void
ViewPort::rotateX(float theta)
{
      if( theta == 0. ) return;
      c = cos(theta*=d2r);
      s = sin(theta);
      rotate2(R[1],R[2]);
      rotate2(R[5],R[6]);
      rotate2(R[9],R[10]);
}
void
ViewPort::rotateY(float theta)
{
   if( theta ==0. ) return;
      c = cos(theta*=d2r);
      s =-sin(theta);
      rotate2(R[0],R[2]);
      rotate2(R[4],R[6]);
      rotate2(R[8],R[10]);
}

void
ViewPort::rotateZ(float theta)
{    if( theta ==0. ) return;
      c = cos(theta*=d2r);
      s = sin(theta);
      rotate2(R[0],R[1]);
      rotate2(R[4],R[5]);
      rotate2(R[8],R[9]);
}    

