//     $Id: Vrml.cxx,v 1.1.1.1 2000/04/03 21:03:58 wilko Exp $
//  Author: T. Burnett
// Project: Arve Graphics

#include "Vrml.h"
#include "ArvePlottableRep.h"

static GraphicsVector currentPoint;
static Hep3Vector origin(0,0,0);
static float eulers[3]={0,0,0};

static const  char * setup_string = 
"#VRML V1.0 ascii\n"
"DEF BackgroundColor Info {  string  \"0.8 0.8 0.8\"  }\n"
"FontStyle { size 3}\n"
"Material { emissiveColor  0.5  0.5  0.5  ambientColor 0 0 0  }\n";

Vrml::Vrml():out(std::cout),_ncoord(0)
{
    out << setup_string;
    center = &origin;
    angles = eulers;
    reset();
    _detail = 0;  //should prevent detail check
}


Vrml::Vrml(std::ostream& c):out(c),_ncoord(0)
{
    out << setup_string;
    center = &origin;
    angles = eulers;
    reset();
    _detail = 0;  //should prevent detail check
}
Vrml::~Vrml(){out.flush();}

Vrml::Vrml(std::ostream& c, const Hep3Vector* _center, float sc,  const float* _angles, const char* t)
: ViewPort(_center,sc,_angles,t),out(c)
{
    //copy: don't setup since already was?? 
    // out << setup_string;
  reset();
}
ViewPort& Vrml::copy(const Hep3Vector* _center, float sc,  const float* _angles, const char* t)
{
    Vrml& v = *new Vrml(out,_center, sc, _angles, t);
   return v;
}
//-----------------------------------------------------------------------------
//                     drawing
void Vrml::set_quad(int /*quad*/, int /*selected*/)
{
 
}
void Vrml::beginSeparator(){   out << "Separator {\n";}
void Vrml::endSeparator()  {   out << "}\n"; _ncoord=0; }

void Vrml::coordinate3(const GraphicsVector* v,int n)
{
   out << "Coordinate3 { point [ \n " ;
   for( int i=0; i<n; i++){
      out << v[i].x()<< ' ' << v[i].y()<< ' '<< v[i].z() ;
      if( i< n-1 ) out <<",\n"	;
   }
   out << "] }\n";
   _ncoord=n;
   currentPoint = v[n-1];
}
void Vrml::indexedLineSet(const int ia[], int n)
{
   out << "IndexedLineSet{ coordIndex[\n";
   for(int i=0; i<n; i++){ 
      out << ia[i] ;
      if( i< n-1 ) out << ",";
   }
   out << "] }\n";
}

void Vrml::indexedFaceSet(const int  *ia,int n)
{
   out << "IndexedFaceSet{ coordIndex[\n";
   for(int i=0; i<n; i++){ 
      out << ia[i] ;
      if( i< n-1 ) out << ",";
   }
   out << "] }\n";
}
void Vrml::lineSet( int n)
{
   out << "IndexedLineSet{ coordIndex[\n";
   for(int i=0; i<n; i++) 
      out << i << ',';
   out << -1 << "] }\n";
}

void Vrml::drawPL(const GraphicsVector* v,int n)
{  
   coordinate3(v,n);
   lineSet(n);
}

void Vrml::drawText(const char *text, const GraphicsVector& pnt, int /*ht*/)
{
   beginSeparator();
   out << "Transform { translation "
       << pnt.x() << ' '<< pnt.y() << ' ' << pnt.z() << "}\n";
   out << "AsciiText { string \"" << text << "\"} \n";
   endSeparator();
}
void Vrml::drawText(const char* text)
{
   drawText(text, currentPoint);

}
 

void Vrml::draw_markers(const GraphicsVector* v,int n)
{ 
   coordinate3(v,n);
   out << "PointSet { startIndex 0 numPoints " << n << "}\n";

}
    // draw markers (points) at the specified points
 
void Vrml::flush(){
  out.flush();
  }
    // flush accumulated drawing commands


//-----------------------------------------------------------------------
//  functions for setting drawing attributes

void Vrml::set_rgb( float r, float g, float b )
{
   out << "Material { emissiveColor "<< r << ' '<< g <<' '<< b << "}\n";
}
void Vrml::set_col_index(int index)
{
    ArvePlottableRep::ColorInfo& cinfo = ArvePlottableRep::pallete[index];
    set_rgb(cinfo.r/255., cinfo.g/255., cinfo.b/255.);
}

void Vrml::set_marker_size(float ){}
void Vrml::set_line_width( float  ){}

void Vrml::set_line_style( int ){}

void Vrml::set_cap_style(int ){}
void Vrml::setJoinStyle(int ){}

void Vrml::setBkColIndex(int ){}
void Vrml::set_enhanced(int){};

//-----------------------------------------------------------------------------
//             stuff associated with projection
/*
void Vrml::setProjection()
{
   glMatrixMode(GL_PROJECTION);
   float projection[]=
     {zoomFactor,0,   0,   0
     ,0,   zoomFactor*aspectRatio,0,   0
     ,0,   0,   -zoomFactor, -zoomFactor*perspective
     ,0,   0,   0,   1};

   glLoadMatrixf(projection);
 
   glMultMatrixf(R);
   
   glScalef(1/scale,1/scale,1/scale);     // overall scale
   glTranslatef(-center->x(),-center->y(),-center->z()); // negative of origin  
   glMatrixMode(GL_MODELVIEW);
}
*/





