
#include "instrument/trackerDetGeo.h"
#include "Event/messageManager.h"

//------- public    -------------------------------

//################################################
trackerDetGeo::trackerDetGeo()
//################################################
{
	createNav();
	createTrk();
	orderLayers();
	m_nameOut = "LAYER";
}
//################################################
trackerDetGeo::~trackerDetGeo()
//################################################
{
	int nEl = m_trackerNav->size();
	for (unsigned int i = 0; i < nEl; i++) delete m_trackerNav->el(i);
}
//################################################
void  trackerDetGeo::draw(ArvePlottableRep& v) 
//################################################
{
	int nlayers = m_trackerNav->size(m_nameOut);
	for (int ilayer= 0; ilayer < nlayers; ilayer ++) {
		m_trackerNav->el("LAYER",ilayer)->draw(v);
	}
}
//################################################
void  trackerDetGeo::writeOut() const
//################################################
{
	if (!messageManager::instance()->acceptLevel("DEBUG_MEDIUM")) return;

	int nlayers = m_trackerNav->size(m_nameOut);
	navigator<detGeo*>* n = m_trackerNav->front("LAYER");
	while (n->navNext() != 0) {
		n->el()->writeOut();
		n = n->navNext();
	}
}

//----------- private --------------------------
//##############################################
void trackerDetGeo::createNav()
//##############################################
{
	m_trackerNav = new navigator<detGeo*>(new detGeo(0,detGeo::PASSIVE,0),"TRACKER");
	m_trackerNav->add(new detGeo(0,detGeo::X,0),"XTRACKER");	
	m_trackerNav->add(new detGeo(0,detGeo::Y,0),"YTRACKER");	
	m_trackerNav->add(new detGeo(0,detGeo::PASSIVE,0),"PBTRACKER");
}
//##############################################
void trackerDetGeo::createTrk()
//##############################################
{
	for (int iview = 0; iview < trackerGeo::numViews(); iview++) {
		detGeo::axis a = (iview == 0? detGeo::X : detGeo::Y);
		for (int ilayer = 0; ilayer < trackerGeo::numLayers(); ilayer++) {
			createSiLayer(ilayer,a);
		}
	}

	for (int ilayer = 0; ilayer < trackerGeo::numPbLayers(); ilayer++) 
		createPbLayer(ilayer);
}
//##############################################
void trackerDetGeo::createSiLayer(int ilayer, detGeo::axis a)
//##############################################
{
		// geometrical position
	int iview = ilayer % 2;
	double zfar = 0;
	if ((iview == 0 && a == detGeo::X) ||
		(iview == 1 && a == detGeo::Y)) zfar = trackerGeo::layerGap(ilayer);
	double zpos = ilayer*trackerGeo::trayHeight()+trackerGeo::Z0()+ zfar;

	int nladders = trackerGeo::nLadders(ilayer, a);
	double size  = nladders*trackerGeo::ladderWidth()+(nladders-1)*trackerGeo::ladderGap();
	double sizeRef  = trackerGeo::trayWidth();
	double pos = 0.5*(size - sizeRef);
	if (fabs(pos) < 1e-5) pos =0.; 

	double xpos = 0.;
	double ypos = 0.;
	if (a == detGeo::X) xpos = pos;
	else ypos = pos;

	double xsize = sizeRef;
	double ysize = sizeRef;
	if (a == detGeo::X) xsize = size;
	else ysize = size;

	Point P(xpos,ypos,zpos);
	Point S(0.5*xsize,0.5*ysize,0.5*trackerGeo::siThickness());

	detGeo* layer = new detGeo(ilayer,a,ilayer,P,S);
	layer->setMaterial("SI",trackerGeo::siX0());
	layer->setName("LAYER");

	getNavigator(a)->add(layer,"LAYER");

	for (int iladder = 0; iladder < trackerGeo::nLadders(ilayer,a); iladder++) 
			createLadder(ilayer,a,iladder);

}
//##############################################
void trackerDetGeo::createPbLayer(int ilayer)
//##############################################
{
	// geometrical position
	int iview = ilayer % 2;
	double zfar = +layerGap(ilayer);   // the lead is always in the far side
	int idlayer = ilayer+trackerGeo::numLayers()-trackerGeo::numPbLayers(); // not the same number of leads
	double zpos = idlayer*trackerGeo::trayHeight() + zfar + trackerGeo::Z0();

	double xpos = 0.;
	double ypos = 0.;

	double sizeRef  = trayWidth();
	double xsize = sizeRef;
	double ysize = sizeRef;
	double zsize = trackerGeo::pbRadLen(idlayer)*trackerGeo::pbX0();

	zpos += 0.5*trackerGeo::siThickness()+0.5*zsize; // lead above

	Point P(xpos,ypos,zpos);
	if (fabs(zsize) < 1e-3) zsize = 0.;
	Point S(0.5*xsize,0.5*ysize,0.5*zsize); 

	detGeo::axis  a = detGeo::PASSIVE;
	detGeo* layer = new detGeo(idlayer,a, idlayer,P,S); 
	layer->setMaterial("PB",pbX0());
	layer->setName("LAYER");
	getNavigator(a)->add(layer,"LAYER");
}

//##############################################
void trackerDetGeo::createLadder(int ilayer, detGeo::axis a, int iladder)
//##############################################
{
	detGeo* layer = getLayer(ilayer,a);
	double zpos = layer->position().z();
	double xpos = layer->position().x();
	double ypos = layer->position().y();

	double zsize = 2.*layer->size().z();

	double pos = -0.5*m_trayWidth+(iladder+0.5)*m_ladderWidth+
		iladder*m_ladderGap;
	if (fabs(pos) < 1e-5) pos =0.; 
	if (a == detGeo::X) xpos = pos;
	else ypos = pos;

	double xsize = m_ladderLength;
	double ysize = m_ladderLength;
	if (a == detGeo::X) xsize = m_ladderWidth;
	else ysize = m_ladderWidth;
	
	Point P(xpos,ypos,zpos);
	Point S(0.5*xsize,0.5*ysize,0.5*zsize);
	// int id = m_ladderAdrress[ilayer][integer(a)][iladder];

	detGeo* ladder = new detGeo(ilayer,a,iladder,P,S);
	ladder->setMaterial("SI",siX0());
	ladder->setName("LADDER");

	getNavigator(ilayer,a)->add(ladder,"LADDER");
	for (int idice = 0; idice < trackerGeo::nDices(ilayer,a,iladder); idice++)
		createDice(ilayer,a,iladder,idice);

}
//##############################################
void trackerDetGeo::createDice(int ilayer, detGeo::axis a, int iladder, int idice)
//##############################################
{
	detGeo* ladder = getLadder(ilayer,a,iladder);
	double xpos = 0.;
	double ypos = 0.;
	double zpos = ladder->position().z();

	double xsize = 0;
	double ysize = 0;
	double zsize = 2.*ladder->size().z();

	double length = 0.;
	if (a == detGeo::X) length = 2.*ladder->size().y();
	else length = 2.*ladder->size().x();

	double diceSize = trackerGeo::diceSize(ilayer,a,iladder);	
	double pos = -0.5*length + (idice+0.5)*diceSize+(idice)*m_ladderInnerGap;
	if (fabs(pos) < 1e-5) pos =0.; 

	if (a == detGeo::X) {
		xpos = ladder->position().x();
		ypos = pos;
		xsize = 2.*ladder->size().x();
		ysize = diceSize;
	} else {
		xpos = pos; 
		ypos = ladder->position().y();
		xsize = diceSize;
		ysize = 2.*ladder->size().y();
	}

	Point P(xpos,ypos,zpos);
	Point S(0.5*xsize,0.5*ysize,0.5*zsize);

	detGeo* dice = new detGeo(ilayer,a,iladder,P,S);
	dice->setMaterial("SI",siX0());
	dice->setName("SIDICE");

	getNavigator(ilayer,a,iladder)->add(dice,"SIDICE");
}
//########################################################
navigator<detGeo*>* trackerDetGeo::getNavigator(detGeo::axis a)  const 
//########################################################
{ 
	if (a == detGeo::X) return m_trackerNav->nav("XTRACKER",0);
	else if (a == detGeo::Y) return m_trackerNav->nav("YTRACKER",0);
	else return m_trackerNav->nav("PBTRACKER",0);
}

//########################################################
navigator<detGeo*>* trackerDetGeo::getNavigator(int ilayer, detGeo::axis a) const
//########################################################
{
	return getNavigator(a)->nav("LAYER",ilayer);
}
//########################################################
navigator<detGeo*>* trackerDetGeo::getNavigator(int ilayer, detGeo::axis a, 
												int iladder) const 
//########################################################
{
	return getNavigator(ilayer,a)->nav("LADDER",iladder);
}
//########################################################
navigator<detGeo*>* trackerDetGeo::getNavigator(int ilayer, detGeo::axis a, 
								 int iladder,int idice)const 
//########################################################
{
	return getNavigator(ilayer,a,iladder)->nav("SIDICE",idice);
}
//######################################################
void trackerDetGeo::orderLayers()
//######################################################
{
	int nLayers = m_trackerNav->size("LAYER");
	for (int ilayer =0; ilayer < nLayers; ilayer++) {
		double z0 = m_trackerNav->el("LAYER",ilayer)->position().z();
		int inext = -1;
		int iprevious = -1;
		double dmax = 1e6;
		double dmin = 1e6;
		for (int jlayer = 0; jlayer < nLayers; jlayer++) {
			if (jlayer != ilayer) { 
				double z = m_trackerNav->el("LAYER",jlayer)->position().z();
				if (z<z0 && fabs(z-z0)< dmin){
					iprevious = jlayer;
					dmin = fabs(z-z0);
				}
				if (z>z0 && fabs(z-z0)<dmax){
					inext = jlayer;
					dmax = fabs(z-z0);
				}
			}
		}
		if (inext>-1) m_trackerNav->nav("LAYER",ilayer)->
			setNext(m_trackerNav->nav("LAYER",inext));
		if (iprevious>-1) m_trackerNav->nav("LAYER",ilayer)->
			setPrev(m_trackerNav->nav("LAYER",iprevious));
	}


}
