#ifndef CsIClusterList_H
#define CsIClusterList_H

#include <vector>
#include "geometry/Point.h"
#include "geometry/Vector.h"
#include "Event/trsDataVI.h"



//----------------------------------------------
//
//   CsICluster
//
//   Transient Storage Data
//----------------------------------------------
//   It contains the high level data for the Calorimeter
//----------------------------------------------
//             J.A Hernando, Santa Cruz 02/29/00
//----------------------------------------------
//#####################################
class CsICluster
//#####################################
{
public:

	// construct
	CsICluster(double e, Point p);
	~CsICluster() {}
	void setDirection(Point v)   {m_direction = v;}
	void setEnergyCorrected(double e) {m_energyCorrected = e;}
	void setEneLayer(std::vector<double> v){m_eneLayer = v;}
	void setPosLayer(std::vector<Vector> v){m_pLayer = v;}
	void setFitEnergy(double e) { m_fitEnergy = e;}
	void setProfChisq(double k) { m_ProfChisq = k;}
	void setCsiAlpha(double a) { m_CsiAlpha =a;}
	void setCsiLambda(double l) { m_CsiLambda = l;}
	void setCsiStart(double s) { m_start = s;}

	// access
	double energySum()        const {return m_energySum;}
	double energyCorrected()  const {return m_energyCorrected;}
	double getEneLayer(int i) const {return m_eneLayer[i];}
	const Vector& getPosLayer(int i) const {return m_pLayer[i];}
	const std::vector<double>& getEneLayer() const {return m_eneLayer;}
	const std::vector<Vector>& getPosLayer() const {return m_pLayer;}
	Point position()          const {return m_position;}
	Vector direction()        const {return m_direction;}
	double getFitEnergy()	  const {return m_fitEnergy;}
	double getProfChisq()	  const {return m_ProfChisq;}
	double getCsiAlpha()	  const {return m_CsiAlpha;}
	double getCsiLambda()	  const {return m_CsiLambda;}
	double getCsiStart()	  const {return m_start;}
	// operations
	void writeOut() const;

protected:

	virtual void ini();

private:

	double m_energySum;
	double m_energyCorrected;
	std::vector<double> m_eneLayer;
	std::vector<Vector> m_pLayer;

	double m_fitEnergy;
	double m_ProfChisq;
	double m_CsiAlpha;
	double m_CsiLambda;
	double m_start;

	Point m_position;
	Vector m_direction;
};

//#####################################
class CsIClusterList : public trsDataVI
//#####################################
{
public:

	CsIClusterList() {clear();}
	~CsIClusterList() {clear();}
	void add(CsICluster* cl) {m_CsIClustersList.push_back(cl);}

	// access
	int num()                  const {return m_CsIClustersList.size();}
	CsICluster* Cluster(int i) const {return m_CsIClustersList[i];}

	//operations
	virtual void clear();
	virtual void make() {}

	virtual void writeOut() const;

protected:

	virtual void ini();

private:

	std::vector<CsICluster*> m_CsIClustersList;
};
#endif	
