//  associated with Tracker for all the evt Status
//
//---------------------------------------------------

#ifndef __SICLUSTERS_H
#define __SICLUSTERS_H 1

#include <vector>
#include "geometry/Point.h"
#include "instrument/trackerGeo.h"
#include "Event/trsDataVI.h"

#define NVIEWS 2
#define NPLANES 16

//----------------------------------------------
//
//   SiClusters
//
//   Transient Storage Data
//----------------------------------------------
//   It contains the Silicon Clusters 
//----------------------------------------------
//             J.A Hernando, Santa Cruz 02/29/00
//----------------------------------------------
//###################################################
class SiCluster
//###################################################
{
public:

	enum view {X,Y,XY};

	friend class SiClusters;

public:

	// construct
	SiCluster(){}
	SiCluster(int id, int v, int ilayer, 
		int istrip0, int istripf, double ToT);
	~SiCluster() {}

	inline void setPosition(Point p)    {m_position = p;}
	inline void setID(int id)    {m_id = id;}
	inline void flag(int flag=1) {m_flag = flag;}
	inline void unflag()         {m_flag = 0;}
	
	// access
	inline int tower()     const {return m_tower;}
	inline int id()        const {return m_id;}
	inline int plane()     const {return m_plane;}
	inline view v()        const {return m_view;}
	inline int chip()      const {return m_chip;}
	inline double strip()  const {return m_strip;}
	bool hitFlagged()      const {return (m_flag!=0);}

	Point position()       const {return m_position;}
	inline double size()   const {return m_size;}

	//operations
	void writeOut() const;
	void draw(ArvePlottableRep& v);

protected:

	void ini();
	
	static enum view intToView(int);
	static int viewToInt(view v);

private:

	int m_tower;
	int m_plane;
	SiCluster::view m_view;
	int m_chip;

	int m_strip0;
	int m_stripf;
	double m_strip;

	double m_size;
	double m_ToT;
	Point m_position;

	int m_id;
	int m_flag;

};

//###################################################
class SiClusters : public trsDataVI
//###################################################
{
public:

	// constructor
	SiClusters()    {ini();}
	~SiClusters()   {clear();}
	void addCluster(SiCluster* cl);

	// access 
	int nHits()  const {return m_clustersList.size();}
	SiCluster* getHit(int i) const {return m_clustersList[i];}

	// access - compatibility
	void flagHit(SiCluster::view v, int id, int iflag=1)   {getHit(v,id)->flag(iflag);}
	void unflagHit(SiCluster::view v, int id)  {getHit(v,id)->unflag();}
    bool hitFlagged(SiCluster::view v, int id) {return getHit(v,id)->hitFlagged();}

	SiCluster* getHit(SiCluster::view v, int id) {return m_clustersList[id];}
	Point const position(SiCluster::view v, int id)   {return getHit(v,id)->position();}  
	double const size(SiCluster::view v, int id)      {return getHit(v,id)->size();}     

	// access
	std::vector<SiCluster*>& getHits(SiCluster::view v, int iplane)
	{return m_clustersByPlaneList[SiCluster::viewToInt(v)][iplane];}
	
	int nHits(SiCluster::view v, int iplane)
	{return (int) getHits(v,iplane).size();}

	void flagHitsInPlane(SiCluster::view v, int iplane);

	// operations
	virtual void clear();
	virtual void make() {}

	void writeOut() const;
	void update(ArvePlottableRep& v)  {draw(v);}

	// utilities
	Point meanHit(SiCluster::view v, int iplane);
	Point meanHitInside(SiCluster::view v, int iplane, double size, Point Pini);
	Point nearestHitOutside(SiCluster::view v, int iplane, double inRadius, 
		Point centerX, int& id);

private:

	virtual void ini();

	void draw(ArvePlottableRep& v);

private:
	
	// the clusters are organized in two lists
	//     One containes the list of all clusters
	//     the other one has them ordered by plane and view to
	//    facilitate access to Pattern Recognition functions

	std::vector<SiCluster*> m_clustersList;
	std::vector<SiCluster*> 
		m_clustersByPlaneList[NVIEWS][NPLANES]; 
};

#endif
