
#include "reconstruction/SiClustersAlg.h"
#include "generation/SiLayers.h"
#include "Event/dataManager.h"
#include "Event/messageManager.h"

//##############################################
void SiClustersAlg::initialize()
//##############################################
{
	if (!defined()) define();
	m_SiClusters->clear();
}
//##############################################
void SiClustersAlg::define()
//##############################################
{
	setDefine(true);
	m_SiLayers      = dataManager::instance()->getData("SiLayers",m_SiLayers);
	m_SiCalibLayers = dataManager::instance()->getData("SiCalibLayers",m_SiCalibLayers);
	m_SiClusters    = dataManager::instance()->getData("SiClusters",m_SiClusters);
}
//##############################################
void SiClustersAlg::execute()
//##############################################
{
	
	// loop in number of layers - conversion to planes!
	int nclusters = 0;
	int nlayers = m_SiLayers->num();
	for (int ilayer = 0; ilayer < nlayers; ilayer++) {
		SiLayer* layer = m_SiLayers->Layer(ilayer);
		int klayer = layer->layer();
		int iview  = layer->view();
		int nhits = layer->nstrips();
		for (int ihit=0; ihit< nhits; ihit++) {
			int istrip0 = layer->idstrip(ihit);
			int istripf = layer->idstrip(ihit);
			if (m_SiCalibLayers->isBadStrip(klayer,iview,istrip0)) continue;
			bool end = false;
			bool found = true;
			for (int jhit = 0; jhit < ihit; jhit++) {
				int jstrip = layer->idstrip(jhit);
				if (m_SiCalibLayers->isBadStrip(klayer,iview,jstrip)) continue;
				if (jstrip == istrip0-1 || jstrip == istripf+1) {
					found = false;
					break;
				}
			}
			if (!found) continue;
			while(!end) {
				end = true;
				for (int jhit=ihit+1; jhit<nhits; jhit++) {
					int jstrip = layer->idstrip(jhit);
					if (m_SiCalibLayers->isBadStrip(klayer,iview,jstrip)) continue;
					if (jstrip == istrip0-1 || jstrip == istripf+1) {
						end = false;
						if (jstrip == istrip0-1) istrip0--;
						if (jstrip == istripf+1) istripf++;
					} 
				}
			}
			int iview = layer->view();
			int ilayer = layer->layer();
			double ToT = layer->ToT(); 
			SiCluster* cl = new SiCluster(nclusters, iview, ilayer,
					 istrip0,istripf,ToT);
			cl->setPosition(position(cl->plane(),cl->v(),cl->strip()));
			m_SiClusters->addCluster(cl);
			nclusters++;
		}
	}
}
//##############################################
void SiClustersAlg::finalize()
//##############################################
{
	m_SiClusters->writeOut();
}
//-------------------- private ----------------------
//###################################################
Point SiClustersAlg::position(int iplane, SiCluster::view v, double strip)
//###################################################
{

	int iladder = (int) strip / trackerGeo::ladderNStrips();
	double stripInLadder = strip - iladder*trackerGeo::ladderNStrips();

	detGeo::axis a = detGeo::X;
	if (v == SiCluster::Y) a = detGeo::Y;

	// note the differences between layers and planes - ordering!
	int ilayer = trackerGeo::numPlanes()-iplane-1;
	trackerDetGeo*   trkGeo   = dataManager::instance()->geo()->tracker();

	detGeo* ladder = trkGeo->getLadder(ilayer, a, iladder);
	
	double Dstrip = ladder->position().x()-ladder->size().x();
	if (v == SiCluster::Y)
		Dstrip = ladder->position().y()-ladder->size().y();
	
	Dstrip += trackerGeo::siDeadDistance();
	Dstrip += stripInLadder*trackerGeo::siStripPitch();

	Point P = ladder->position();
	double x = P.x();
	double y = P.y();
	double z = P.z();
	if (v == SiCluster::X) x = Dstrip;
	else y = Dstrip;

	P = Point(x,y,z);
	return P;
}
