//     $Id: WinGUI.h,v 1.1.1.1 2000/04/03 21:04:06 wilko Exp $
//  Author: T. Burnett
// Project: Arve graphics
//
// Win GUI definition.

#ifndef WinGUI_H
#define WinGUI_H

#include "graphics/GUI.h"
#if !defined (WIN32) && !defined(_WINDOWS)
#  error "This code only compiles for the Windows NT or Window95"
#endif
#define STRICT
#include <windows.h>
#undef min
#undef max
#include <vector>

#pragma warning(disable:4786)

// all that's needed for user main (here as an example)
#if defined( _Windows ) || defined(WIN32)
extern "C" void UserMain(int argc, char *argv[]);  /* needed to start up user's program */
#define main(ARGC,ARGV) UserMain(ARGC,ARGV)
#endif

class WinScene;

class WinGUI : public GUI {
    // Windows 95/NT implementation of GUI functions
public:
    WinGUI(const char* application="Win32", const char* title=0);
    ~WinGUI();
    
    void addToMenu(const char* title, Command* command);
    Menu* beginPullDownMenu(const char*, Menu* );
    void addToggleToMenu(const char* title,int state, Command* set, Command* unset);
    void restorePullDownMenu(Menu*);
    void endPullDownMenu();
    void menuSeparator();
    // handle menus
    
    SceneControl* graphicsWindow(float size);
    GUIostream* textWindow(const char* name);
    // create, return handle to associated window
    
    void run();
    // start the window system
    
    void setTitle(const char* newtitle);
    void quit();
    
    void processMessages();
    
    char * askForFileName(const char* a, const char* b, const char* c);
    char* askUser(const char* promptString, const char* defaultString);
    void inform(const char* msg);
    
    const char* nameOf()const{return "WinGUI";}
    
    void execute(WPARAM);
    // execute the given command, needed by the Windows message handling procedure
    
    // following are static and public for communication with windows routines
    static HINSTANCE s_instance; // handle to the application instance. Set by windows main 
    static WinGUI* s_gui;  //pointer to instance 
    static HWND s_hwnd;     // windows handle to the main window
    static WinScene* s_graphics_window; // primary (3d display) window
    static WinScene* s_2d_window; // secondary (for plots) window
    
private:
    enum{ cmdOffset=100}; // constant for command indeces

    HMENU m_currentMenu;
    HMENU m_menuBar;

    UINT m_menuId;
    
    typedef vector<Command* > CommandList;
    CommandList m_commands;
    
    GUIostream* m_text_window;
    SceneControl* m_graphics_window;

    CommandList m_owned_commands; // separate list for deleting the extra toggle commands
    
    class Toggle; // used in implementation
};
#endif

